@@ -33,15 +33,18 @@
 #include "KoShapeLayer.h"
 #include "KoShapeRegistry.h"
 #include "KoShapeManager.h"
+#include "KoSelectedShapesProxy.h"
 #include "KoCanvasBase.h"
 #include "KoInputDeviceHandlerRegistry.h"
 #include "KoInputDeviceHandlerEvent.h"
 #include "KoPointerEvent.h"
 #include "tools/KoCreateShapesTool.h"
 #include "tools/KoZoomTool.h"
-#include "tools/KoPanTool.h"
 #include "kis_action_registry.h"
 #include "KoToolFactoryBase.h"
+#include "kis_assert.h"
+
+#include <krita_container_utils.h>
 
 // Qt + kde
 #include <QWidget>
@@ -72,11 +75,11 @@ class CanvasData
 {
 public:
     CanvasData(KoCanvasController *cc, const KoInputDevice &id)
-            : activeTool(0),
-            canvas(cc),
-            inputDevice(id),
-            dummyToolWidget(0),
-            dummyToolLabel(0)
+        : activeTool(0),
+          canvas(cc),
+          inputDevice(id),
+          dummyToolWidget(0),
+          dummyToolLabel(0)
     {
     }
 
@@ -88,90 +91,96 @@ public:
 
     void activateToolActions()
     {
-        disabledDisabledActions.clear();
-        disabledActions.clear();
-        disabledCanvasShortcuts.clear();
-        // we do several things here
-        // 1. enable the actions of the active tool
-        // 2. disable conflicting actions
-        // 3. replace conflicting actions in the action collection
-        KActionCollection *canvasActionCollection = canvas->actionCollection();
+        toolActions.clear();
+        disabledGlobalActions.clear();
 
-        QHash<QString, QAction *> toolActions = activeTool->actions();
-        QHash<QString, QAction *>::const_iterator it(toolActions.constBegin());
+        KActionCollection *windowActionCollection = canvas->actionCollection();
 
-        for (; it != toolActions.constEnd(); ++it) {
-            if (canvasActionCollection) {
-
-                QString toolActionID = it.key();
-                QAction *toolAction = it.value();
-
-                QAction * action = qobject_cast<QAction*>(canvasActionCollection->action(it.key()));
-                if (action) {
-                    canvasActionCollection->takeAction(action);
-                    if (action != it.value()) {
-                        if (action->isEnabled()) {
-                            action->setEnabled(false);
-                            disabledActions.append(action);
-                        } else  {
-                            disabledDisabledActions.append(action);
-                        }
-                    }
-                }
-                Q_FOREACH (QAction *a, canvasActionCollection->actions()) {
-                    QAction *canvasAction = dynamic_cast<QAction*>(a);
-                    if (canvasAction && canvasAction->shortcut().toString() != "" && canvasAction->shortcut() == toolAction->shortcut()) {
-                        warnFlake << activeToolId << ": action" << toolActionID << "conflicts with canvas action" << canvasAction->objectName() << "shortcut:" << canvasAction->shortcut().toString();
-                        disabledCanvasShortcuts[canvasAction] = canvasAction->shortcut().toString();
-                        canvasAction->setShortcut(QKeySequence());
-                    }
-                }
-                canvasActionCollection->addAction(toolActionID, toolAction);
-            }
-            it.value()->setEnabled(true);
+        if (!windowActionCollection) {
+            qWarning() << "We haven't got an action collection";
+            return;
         }
-        canvasActionCollection->readSettings(); // The shortcuts might have been configured in the meantime.
+
+        QStringList globalActions;
+
+        QMap<QKeySequence, QStringList> shortcutMap;
+
+        //qDebug() << "................... activating tool" << activeToolId;
+
+        Q_FOREACH(QAction *action, windowActionCollection->actions()) {
+
+            //qDebug() << "Action" << action->objectName() << "shortcuts" << action->shortcuts();
+            if (action->property("tool_action").isValid()) {
+                QStringList tools = action->property("tool_action").toStringList();
+                //qDebug() << "\tassociated with" << tools;
+                if (tools.contains(activeToolId)) {
+                    //qDebug() << "\t\tenabling";
+                    action->setEnabled(true);
+                    toolActions << action->objectName();
+                }
+                else {
+                    action->setDisabled(true);
+                }
+            }
+            else {
+                globalActions << action->objectName();
+            }
+
+            Q_FOREACH(QKeySequence keySequence, action->shortcuts()) {
+                // After loading a custom shortcut profile, shortcuts can be defined as an empty string, which is not an empty shortcut
+                if (keySequence.toString() != "") {
+                    if (shortcutMap.contains(keySequence)) {
+                        shortcutMap[keySequence].append(action->objectName());
+                    }
+                    else {
+                        shortcutMap[keySequence] = QStringList() << action->objectName();
+                    }
+                }
+            }
+        }
+
+        // Make sure the tool's actions override the global actions that aren't associated with the tool.
+        Q_FOREACH(const QKeySequence &k, shortcutMap.keys()) {
+            if (shortcutMap[k].size() > 1) {
+                QStringList actions = shortcutMap[k];
+                //qDebug() << k << actions;
+                bool toolActionFound = false;
+                Q_FOREACH(const QString &action, actions) {
+                    if (toolActions.contains(action)) {
+                        toolActionFound = true;
+                    }
+                }
+                Q_FOREACH(const QString &action, actions) {
+                    if (toolActionFound && globalActions.contains(action)) {
+                        //qDebug() << "\tdisabling global action" << action;
+                        windowActionCollection->action(action)->setEnabled(false);
+                        disabledGlobalActions << action;
+                    }
+                }
+                //qDebug() << k << shortcutMap[k];
+            }
+        }
+
+        windowActionCollection->readSettings(); // The shortcuts might have been configured in the meantime.
     }
 
     void deactivateToolActions()
     {
-
         if (!activeTool)
             return;
-        // disable actions of active tool
-        Q_FOREACH (QAction *action, activeTool->actions()) {
-            action->setEnabled(false);
-        }
 
-        // enable actions which where disabled on activating the active tool
-        // and re-add them to the action collection
-        KActionCollection *ac = canvas->actionCollection();
-        Q_FOREACH (QPointer<QAction> action, disabledDisabledActions) {
-            if (action) {
-                if (ac) {
-                    ac->addAction(action->objectName(), action);
-                }
-            }
-        }
-        disabledDisabledActions.clear();
+        //qDebug() << "............... deactivating previous tool because activating" << activeToolId;
 
-        Q_FOREACH (QPointer<QAction> action, disabledActions) {
-            if (action) {
-                action->setEnabled(true);
-                if(ac) {
-                    ac->addAction(action->objectName(), action);
-                }
-            }
-        }
-        disabledActions.clear();
+        KActionCollection *windowActionCollection = canvas->actionCollection();
 
-        QMap<QPointer<QAction>, QString>::const_iterator it(disabledCanvasShortcuts.constBegin());
-        for (; it != disabledCanvasShortcuts.constEnd(); ++it) {
-            QAction *action = it.key();
-            QString shortcut = it.value();
-            action->setShortcut(shortcut);
+        Q_FOREACH(const QString &action, toolActions) {
+            //qDebug() << "disabling" << action;
+            windowActionCollection->action(action)->setDisabled(true);
+        }
+        Q_FOREACH(const QString &action, disabledGlobalActions) {
+            //qDebug() << "enabling" << action;
+            windowActionCollection->action(action)->setEnabled(true);
         }
-        disabledCanvasShortcuts.clear();
     }
 
     KoToolBase *activeTool;     // active Tool
@@ -183,19 +192,18 @@ public:
     const KoInputDevice inputDevice;
     QWidget *dummyToolWidget;  // the widget shown in the toolDocker.
     QLabel *dummyToolLabel;
-    QList<QPointer<QAction> > disabledActions; ///< disabled conflicting actions
-    QList<QPointer<QAction> > disabledDisabledActions; ///< disabled conflicting actions that were already disabled
-    QMap<QPointer<QAction>, QString> disabledCanvasShortcuts; ///< Shortcuts that were temporarily removed from canvas actions because the tool overrides
+    QStringList toolActions;
+    QStringList disabledGlobalActions;
 };
 
 
 // ******** KoToolManager **********
 KoToolManager::KoToolManager()
     : QObject(),
-    d(new Private(this))
+      d(new Private(this))
 {
-    connect(QApplication::instance(), SIGNAL(focusChanged(QWidget*, QWidget*)),
-            this, SLOT(movedFocus(QWidget*, QWidget*)));
+    connect(QApplication::instance(), SIGNAL(focusChanged(QWidget*,QWidget*)),
+            this, SLOT(movedFocus(QWidget*,QWidget*)));
 }
 
 KoToolManager::~KoToolManager()
@@ -244,17 +252,6 @@ void KoToolManager::registerToolActions(KActionCollection *ac, KoCanvasControlle
         return;
     }
 
-    // Actions available during the use of individual tools
-    CanvasData *cd = d->canvasses.value(controller).first();
-    Q_FOREACH (KoToolBase *tool, cd->allTools) {
-        QHash<QString, QAction*> actions = tool->actions();
-        QHash<QString, QAction*>::const_iterator action(actions.constBegin());
-        for (; action != actions.constEnd(); ++action) {
-            if (!ac->action(action.key()))
-                ac->addAction(action.key(), action.value());
-        }
-    }
-
     // Actions used to switch tools via shortcuts
     Q_FOREACH (ToolHelper * th, d->tools) {
         if (ac->action(th->id())) {
@@ -293,19 +290,6 @@ void KoToolManager::attemptCanvasControllerRemoval(QObject* controller)
     }
 }
 
-void KoToolManager::updateShapeControllerBase(KoShapeBasedDocumentBase *shapeController, KoCanvasController *canvasController)
-{
-    if (!d->canvasses.contains(canvasController))
-        return;
-
-    QList<CanvasData *> canvasses = d->canvasses[canvasController];
-    Q_FOREACH (CanvasData *canvas, canvasses) {
-        Q_FOREACH (KoToolBase *tool, canvas->allTools.values()) {
-            tool->updateShapeController(shapeController);
-        }
-    }
-}
-
 void KoToolManager::switchToolRequested(const QString & id)
 {
     Q_ASSERT(d->canvasData);
@@ -345,7 +329,7 @@ KoCreateShapesTool * KoToolManager::shapeCreatorTool(KoCanvasBase *canvas) const
     Q_FOREACH (KoCanvasController *controller, d->canvasses.keys()) {
         if (controller->canvas() == canvas) {
             KoCreateShapesTool *createTool = dynamic_cast<KoCreateShapesTool*>
-                                             (d->canvasData->allTools.value(KoCreateShapesTool_ID));
+                    (d->canvasData->allTools.value(KoCreateShapesTool_ID));
             Q_ASSERT(createTool /* ID changed? */);
             return createTool;
         }
@@ -372,26 +356,27 @@ KoCanvasController *KoToolManager::activeCanvasController() const
 
 QString KoToolManager::preferredToolForSelection(const QList<KoShape*> &shapes)
 {
-    QList<QString> types;
-    Q_FOREACH (KoShape *shape, shapes)
-        if (! types.contains(shape->shapeId()))
-            types.append(shape->shapeId());
+    QSet<QString> shapeTypes;
+    Q_FOREACH (KoShape *shape, shapes) {
+        shapeTypes << shape->shapeId();
+    }
+    //KritaUtils::makeContainerUnique(types);
 
     QString toolType = KoInteractionTool_ID;
     int prio = INT_MAX;
     Q_FOREACH (ToolHelper *helper, d->tools) {
+        if (helper->id() == KoCreateShapesTool_ID) continue;
         if (helper->priority() >= prio)
             continue;
-        if (helper->toolType() == KoToolFactoryBase::mainToolType())
-            continue;
 
         bool toolWillWork = false;
-        foreach (const QString &type, types) {
+        foreach (const QString &type, shapeTypes) {
             if (helper->activationShapeId().split(',').contains(type)) {
                 toolWillWork = true;
                 break;
             }
         }
+
         if (toolWillWork) {
             toolType = helper->id();
             prio = helper->priority();
@@ -400,52 +385,6 @@ QString KoToolManager::preferredToolForSelection(const QList<KoShape*> &shapes)
     return toolType;
 }
 
-void KoToolManager::injectDeviceEvent(KoInputDeviceHandlerEvent * event)
-{
-    if (d->canvasData && d->canvasData->canvas->canvas()) {
-        if (static_cast<KoInputDeviceHandlerEvent::Type>(event->type()) == KoInputDeviceHandlerEvent::ButtonPressed)
-            d->canvasData->activeTool->customPressEvent(event->pointerEvent());
-        else if (static_cast<KoInputDeviceHandlerEvent::Type>(event->type()) == KoInputDeviceHandlerEvent::ButtonReleased)
-            d->canvasData->activeTool->customReleaseEvent(event->pointerEvent());
-        else if (static_cast<KoInputDeviceHandlerEvent::Type>(event->type()) ==  KoInputDeviceHandlerEvent::PositionChanged)
-            d->canvasData->activeTool->customMoveEvent(event->pointerEvent());
-    }
-}
-
-void KoToolManager::addDeferredToolFactory(KoToolFactoryBase *toolFactory)
-{
-    ToolHelper *tool = new ToolHelper(toolFactory);
-    // make sure all plugins are loaded as otherwise we will not load them
-    d->setup();
-    d->tools.append(tool);
-
-    // connect to all tools so we can hear their button-clicks
-    connect(tool, SIGNAL(toolActivated(ToolHelper*)), this, SLOT(toolActivated(ToolHelper*)));
-
-    // now create tools for all existing canvases
-    Q_FOREACH (KoCanvasController *controller, d->canvasses.keys()) {
-
-        // this canvascontroller is unknown, which is weird
-        if (!d->canvasses.contains(controller)) {
-            continue;
-        }
-
-        // create a tool for all canvasdata objects (i.e., all input devices on this canvas)
-        foreach (CanvasData *cd, d->canvasses[controller]) {
-            QPair<QString, KoToolBase*> toolPair = createTools(controller, tool);
-            if (toolPair.second) {
-                cd->allTools.insert(toolPair.first, toolPair.second);
-            }
-        }
-
-        // Then create a button for the toolbox for this canvas
-        if (tool->id() == KoCreateShapesTool_ID) {
-            continue;
-        }
-
-        emit addedTool(tool->toolAction(), controller);
-    }
-}
 
 QPair<QString, KoToolBase*> KoToolManager::createTools(KoCanvasController *controller, ToolHelper *tool)
 {
@@ -466,13 +405,7 @@ QPair<QString, KoToolBase*> KoToolManager::createTools(KoCanvasController *contr
     KoToolBase *tl = tool->createTool(controller->canvas());
     if (tl) {
         d->uniqueToolIds.insert(tl, tool->uniqueId());
-
         tl->setObjectName(tool->id());
-
-        Q_FOREACH (QAction *action, tl->actions()) {
-            action->setEnabled(false);
-        }
-
     }
 
     KoZoomTool *zoomTool = dynamic_cast<KoZoomTool*>(tl);
@@ -480,26 +413,13 @@ QPair<QString, KoToolBase*> KoToolManager::createTools(KoCanvasController *contr
         zoomTool->setCanvasController(controller);
     }
 
-    KoPanTool *panTool = dynamic_cast<KoPanTool*>(tl);
-    if (panTool) {
-        panTool->setCanvasController(controller);
-    }
-
     return QPair<QString, KoToolBase*>(tool->id(), tl);
 }
 
 
-// NOT IMPLEMENTED
-void KoToolManager::updateToolShortcuts()
+void KoToolManager::initializeCurrentToolForCanvas()
 {
-    // auto actionRegistry = KisActionRegistry::instance();
-    // foreach (KoToolBase *t, allTools) {
-    //     for (auto it = t->actions().constBegin();
-    //          it != t->actions().constEnd();
-    //          ++it;) {
-    //         actionRegistry->updateShortcut(it.key(), it.value());
-    //     }
-    // }
+    d->postSwitchTool(false);
 }
 
 KoToolManager* KoToolManager::instance()
@@ -524,8 +444,8 @@ KoToolManager::Private *KoToolManager::priv()
 
 KoToolManager::Private::Private(KoToolManager *qq)
     : q(qq),
-    canvasData(0),
-    layerExplicitlyDisabled(false)
+      canvasData(0),
+      layerExplicitlyDisabled(false)
 {
 }
 
@@ -545,7 +465,7 @@ CanvasData *KoToolManager::Private::createCanvasData(KoCanvasController *control
         }
     }
     KoCreateShapesTool *createShapesTool = dynamic_cast<KoCreateShapesTool*>(toolsHash.value(KoCreateShapesTool_ID));
-    Q_ASSERT(createShapesTool);
+    KIS_ASSERT(createShapesTool);
     QString id = KoShapeRegistry::instance()->keys()[0];
     createShapesTool->setShapeId(id);
 
@@ -577,15 +497,15 @@ void KoToolManager::Private::setup()
 void KoToolManager::Private::connectActiveTool()
 {
     if (canvasData->activeTool) {
-        connect(canvasData->activeTool, SIGNAL(cursorChanged(const QCursor &)),
-                q, SLOT(updateCursor(const QCursor &)));
-        connect(canvasData->activeTool, SIGNAL(activateTool(const QString &)),
-                q, SLOT(switchToolRequested(const QString &)));
-        connect(canvasData->activeTool, SIGNAL(activateTemporary(const QString &)),
-                q, SLOT(switchToolTemporaryRequested(const QString &)));
+        connect(canvasData->activeTool, SIGNAL(cursorChanged(QCursor)),
+                q, SLOT(updateCursor(QCursor)));
+        connect(canvasData->activeTool, SIGNAL(activateTool(QString)),
+                q, SLOT(switchToolRequested(QString)));
+        connect(canvasData->activeTool, SIGNAL(activateTemporary(QString)),
+                q, SLOT(switchToolTemporaryRequested(QString)));
         connect(canvasData->activeTool, SIGNAL(done()), q, SLOT(switchBackRequested()));
-        connect(canvasData->activeTool, SIGNAL(statusTextChanged(const QString &)),
-                q, SIGNAL(changedStatusText(const QString &)));
+        connect(canvasData->activeTool, SIGNAL(statusTextChanged(QString)),
+                q, SIGNAL(changedStatusText(QString)));
     }
 
     // we expect the tool to emit a cursor on activation.
@@ -600,16 +520,17 @@ void KoToolManager::Private::disconnectActiveTool()
         canvasData->deactivateToolActions();
         // repaint the decorations before we deactivate the tool as it might deleted
         // data needed for the repaint
+        emit q->aboutToChangeTool(canvasData->canvas);
         canvasData->activeTool->deactivate();
-        disconnect(canvasData->activeTool, SIGNAL(cursorChanged(const QCursor&)),
-                   q, SLOT(updateCursor(const QCursor&)));
-        disconnect(canvasData->activeTool, SIGNAL(activateTool(const QString &)),
-                   q, SLOT(switchToolRequested(const QString &)));
-        disconnect(canvasData->activeTool, SIGNAL(activateTemporary(const QString &)),
-                   q, SLOT(switchToolTemporaryRequested(const QString &)));
+        disconnect(canvasData->activeTool, SIGNAL(cursorChanged(QCursor)),
+                   q, SLOT(updateCursor(QCursor)));
+        disconnect(canvasData->activeTool, SIGNAL(activateTool(QString)),
+                   q, SLOT(switchToolRequested(QString)));
+        disconnect(canvasData->activeTool, SIGNAL(activateTemporary(QString)),
+                   q, SLOT(switchToolTemporaryRequested(QString)));
         disconnect(canvasData->activeTool, SIGNAL(done()), q, SLOT(switchBackRequested()));
-        disconnect(canvasData->activeTool, SIGNAL(statusTextChanged(const QString &)),
-                   q, SIGNAL(changedStatusText(const QString &)));
+        disconnect(canvasData->activeTool, SIGNAL(statusTextChanged(QString)),
+                   q, SIGNAL(changedStatusText(QString)));
     }
 
     // emit a empty status text to clear status text from last active tool
@@ -688,14 +609,7 @@ void KoToolManager::Private::postSwitchTool(bool temporary)
         KoSelection *selection = canvasData->activeTool->canvas()->shapeManager()->selection();
         Q_ASSERT(selection);
 
-        Q_FOREACH (KoShape *shape, selection->selectedShapes()) {
-            QSet<KoShape*> delegates = shape->toolDelegates();
-            if (delegates.isEmpty()) { // no delegates, just the orig shape
-                shapesToOperateOn << shape;
-            } else {
-                shapesToOperateOn += delegates;
-            }
-        }
+        shapesToOperateOn = QSet<KoShape*>::fromList(selection->selectedEditableShapesAndDelegates());
     }
 
     if (canvasData->canvas->canvas()) {
@@ -739,10 +653,7 @@ void KoToolManager::Private::postSwitchTool(bool temporary)
 
     emit q->changedTool(canvasData->canvas, uniqueToolIds.value(canvasData->activeTool));
 
-    KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
-    if (canvasControllerWidget) {
-        canvasControllerWidget->setToolOptionWidgets(optionWidgetList);
-    }
+    emit q->toolOptionWidgetsChanged(canvasData->canvas, optionWidgetList);
 }
 
 
@@ -816,10 +727,7 @@ void KoToolManager::Private::detachCanvas(KoCanvasController *controller)
         if (newCanvas) {
             switchCanvasData(canvasses.value(newCanvas).first());
         } else {
-            KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
-            if (canvasControllerWidget) {
-                canvasControllerWidget->setToolOptionWidgets(QList<QPointer<QWidget> >());
-            }
+            emit q->toolOptionWidgetsChanged(controller, QList<QPointer<QWidget> >());
             // as a last resort just set a blank one
             canvasData = 0;
         }
@@ -868,7 +776,7 @@ void KoToolManager::Private::attachCanvas(KoCanvasController *controller)
         int highestPriority = INT_MAX;
         ToolHelper * helper = 0;
         Q_FOREACH (ToolHelper * th, tools) {
-            if (th->toolType() == KoToolFactoryBase::mainToolType()) {
+            if (th->section() == KoToolFactoryBase::mainToolType()) {
                 if (th->priority() < highestPriority) {
                     highestPriority = qMin(highestPriority, th->priority());
                     helper = th;
@@ -882,7 +790,7 @@ void KoToolManager::Private::attachCanvas(KoCanvasController *controller)
     Connector *connector = new Connector(controller->canvas()->shapeManager());
     connect(connector, SIGNAL(selectionChanged(QList<KoShape*>)), q,
             SLOT(selectionChanged(QList<KoShape*>)));
-    connect(controller->canvas()->shapeManager()->selection(),
+    connect(controller->canvas()->selectedShapesProxy(),
             SIGNAL(currentLayerChanged(const KoShapeLayer*)),
             q, SLOT(currentLayerChanged(const KoShapeLayer*)));
 
@@ -916,7 +824,7 @@ void KoToolManager::Private::movedFocus(QWidget *from, QWidget *to)
     // if the 'to' is one of our canvasWidgets, then switch.
 
     // for code simplicity the current canvas will be checked again,
-    // but would have been catched already in the lines above, so no issue
+    // but would have been caught already in the lines above, so no issue
     KoCanvasController *newCanvas = 0;
     Q_FOREACH (KoCanvasController* canvas, canvasses.keys()) {
         if (canvas->canvas()->canvasWidget() == to) {
@@ -970,13 +878,16 @@ void KoToolManager::Private::selectionChanged(const QList<KoShape*> &shapes)
     // there needs to be at least one shape for a tool without an activationShapeId
     // to work
     // if not change the current tool to the default tool
+
+    const QStringList activationShapeIds = canvasData->activationShapeId.split(',');
+
     if (!(canvasData->activationShapeId.isNull() && shapes.size() > 0)
-                && canvasData->activationShapeId != "flake/always"
-                && canvasData->activationShapeId != "flake/edit") {
+            && !activationShapeIds.contains("flake/always")
+            && !activationShapeIds.contains("flake/edit")) {
 
         bool currentToolWorks = false;
         foreach (const QString &type, types) {
-            if (canvasData->activationShapeId.split(',').contains(type)) {
+            if (activationShapeIds.contains(type)) {
                 currentToolWorks = true;
                 break;
             }
@@ -992,7 +903,7 @@ void KoToolManager::Private::selectionChanged(const QList<KoShape*> &shapes)
 void KoToolManager::Private::currentLayerChanged(const KoShapeLayer *layer)
 {
     emit q->currentLayerChanged(canvasData->canvas, layer);
-    layerExplicitlyDisabled = layer && !layer->isEditable();
+    layerExplicitlyDisabled = layer && !layer->isShapeEditable();
     updateToolForProxy();
 
     debugFlake << "Layer changed to" << layer << "explicitly disabled:" << layerExplicitlyDisabled;
@@ -1023,15 +934,6 @@ void KoToolManager::Private::switchInputDevice(const KoInputDevice &device)
 
     QList<CanvasData*> items = canvasses[canvasData->canvas];
 
-    // disable all actions for all tools in the all canvasdata objects for this canvas.
-    Q_FOREACH (CanvasData *cd, items) {
-        Q_FOREACH (KoToolBase* tool, cd->allTools) {
-            Q_FOREACH (QAction * action, tool->actions()) {
-                action->setEnabled(false);
-            }
-        }
-    }
-
     // search for a canvasdata object for the current input device
     Q_FOREACH (CanvasData *cd, items) {
         if (cd->inputDevice == device) {
@@ -1069,16 +971,5 @@ void KoToolManager::Private::registerToolProxy(KoToolProxy *proxy, KoCanvasBase
     }
 }
 
-void KoToolManager::Private::switchToolByShortcut(QKeyEvent *event)
-{
-    QKeySequence item(event->key() | ((Qt::ControlModifier | Qt::AltModifier) & event->modifiers()));
-
-    if (event->key() == Qt::Key_Space && event->modifiers() == 0) {
-        switchTool(KoPanTool_ID, true);
-    } else if (event->key() == Qt::Key_Escape && event->modifiers() == 0) {
-        switchTool(KoInteractionTool_ID, false);
-    }
-}
-
 //have to include this because of Q_PRIVATE_SLOT
 #include "moc_KoToolManager.cpp"
