@@ -43,7 +43,6 @@
 #include "KoShapeManager.h"
 #include "KoSelection.h"
 #include "KoShapeLayer.h"
-#include "KoShapePaste.h"
 #include "KoShapeRegistry.h"
 #include "KoShapeController.h"
 #include "KoOdf.h"
@@ -52,15 +51,9 @@
 
 
 KoToolProxyPrivate::KoToolProxyPrivate(KoToolProxy *p)
-    : activeTool(0),
-    tabletPressed(false),
-    hasSelection(false),
-    controller(0),
-    parent(p)
+    : parent(p)
 {
     scrollTimer.setInterval(100);
-    mouseLeaveWorkaround = false;
-    multiClickCount = 0;
 }
 
 void KoToolProxyPrivate::timeout() // Auto scroll the canvas
@@ -96,8 +89,10 @@ void KoToolProxyPrivate::checkAutoScroll(const KoPointerEvent &event)
     if (!activeTool) return;
     if (!activeTool->wantsAutoScroll()) return;
     if (!event.isAccepted()) return;
+    if (!isToolPressed) return;
     if (event.buttons() != Qt::LeftButton) return;
 
+
     widgetScrollPoint = event.pos();
 
     if (! scrollTimer.isActive())
@@ -119,14 +114,14 @@ bool KoToolProxyPrivate::isActiveLayerEditable()
 
     KoShapeManager * shapeManager = activeTool->canvas()->shapeManager();
     KoShapeLayer * activeLayer = shapeManager->selection()->activeLayer();
-    if (activeLayer && !activeLayer->isEditable())
+    if (activeLayer && !activeLayer->isShapeEditable())
         return false;
     return true;
 }
 
 KoToolProxy::KoToolProxy(KoCanvasBase *canvas, QObject *parent)
-        : QObject(parent),
-        d(new KoToolProxyPrivate(this))
+    : QObject(parent),
+      d(new KoToolProxyPrivate(this))
 {
     KoToolManager::instance()->priv()->registerToolProxy(this, canvas);
 
@@ -163,65 +158,6 @@ KoCanvasBase* KoToolProxy::canvas() const
     return d->controller->canvas();
 }
 
-void KoToolProxy::touchEvent(QTouchEvent *event)
-{
-    QPointF point;
-    QList<KoTouchPoint> touchPoints;
-
-    bool isPrimary = true;
-    Q_FOREACH (QTouchEvent::TouchPoint p, event->touchPoints()) {
-        QPointF docPoint = widgetToDocument(p.screenPos());
-        if (isPrimary) {
-            point = docPoint;
-            isPrimary = false;
-        }
-        KoTouchPoint touchPoint;
-        touchPoint.touchPoint = p;
-        touchPoint.point = point;
-        touchPoint.lastPoint = widgetToDocument(p.lastNormalizedPos());
-        touchPoints << touchPoint;
-    }
-
-    KoPointerEvent ev(event, point, touchPoints);
-
-    KoInputDevice id;
-    KoToolManager::instance()->priv()->switchInputDevice(id);
-
-    switch (event->type()) {
-    case QEvent::TouchBegin:
-        ev.setTabletButton(Qt::LeftButton);
-        if (d->activeTool) {
-            if( d->activeTool->wantsTouch() )
-                d->activeTool->touchEvent(event);
-            else
-                d->activeTool->mousePressEvent(&ev);
-        }
-        break;
-    case QEvent::TouchUpdate:
-        ev.setTabletButton(Qt::LeftButton);
-        if (d->activeTool) {
-            if( d->activeTool->wantsTouch() )
-                d->activeTool->touchEvent(event);
-            else
-                d->activeTool->mouseMoveEvent(&ev);
-        }
-        break;
-    case QEvent::TouchEnd:
-        ev.setTabletButton(Qt::LeftButton);
-        if (d->activeTool) {
-            if( d->activeTool->wantsTouch() )
-                d->activeTool->touchEvent(event);
-            else
-                d->activeTool->mouseReleaseEvent(&ev);
-        }
-        break;
-    default:
-        ; // ingore the rest
-    }
-    d->mouseLeaveWorkaround = true;
-
-}
-
 void KoToolProxy::tabletEvent(QTabletEvent *event, const QPointF &point)
 {
     // We get these events exclusively from KisToolProxy - accept them
@@ -265,13 +201,27 @@ void KoToolProxy::mousePressEvent(KoPointerEvent *ev)
     KoToolManager::instance()->priv()->switchInputDevice(id);
     d->mouseDownPoint = ev->pos();
 
-    if (d->tabletPressed) // refuse to send a press unless there was a release first.
+
+    // this tries to make sure another mouse press event doesn't happen
+    // before a release event happens
+    if (d->isToolPressed) {
+        mouseReleaseEvent(ev);
+        d->tabletPressed = false;
+        d->scrollTimer.stop();
+
+        if (d->activeTool) {
+            d->activeTool->mouseReleaseEvent(ev);
+        }
+
+        d->isToolPressed = false;
+
         return;
+    }
 
     QPointF globalPoint = ev->globalPos();
     if (d->multiClickGlobalPoint != globalPoint) {
         if (qAbs(globalPoint.x() - d->multiClickGlobalPoint.x()) > 5||
-            qAbs(globalPoint.y() - d->multiClickGlobalPoint.y()) > 5) {
+                qAbs(globalPoint.y() - d->multiClickGlobalPoint.y()) > 5) {
             d->multiClickCount = 0;
         }
         d->multiClickGlobalPoint = globalPoint;
@@ -303,6 +253,8 @@ void KoToolProxy::mousePressEvent(KoPointerEvent *ev)
         d->multiClickCount = 0;
         ev->ignore();
     }
+
+    d->isToolPressed = true;
 }
 
 void KoToolProxy::mousePressEvent(QMouseEvent *event, const QPointF &point)
@@ -319,29 +271,14 @@ void KoToolProxy::mouseDoubleClickEvent(QMouseEvent *event, const QPointF &point
 
 void KoToolProxy::mouseDoubleClickEvent(KoPointerEvent *event)
 {
-     // let us handle it as any other mousepress (where we then detect multi clicks
+    // let us handle it as any other mousepress (where we then detect multi clicks
     mousePressEvent(event);
-    if (!event->isAccepted() && d->activeTool)
-        d->activeTool->canvas()->shapeManager()->suggestChangeTool(event);
 }
 
 void KoToolProxy::mouseMoveEvent(QMouseEvent *event, const QPointF &point)
 {
-    if (d->mouseLeaveWorkaround) {
-        d->mouseLeaveWorkaround = false;
-        return;
-    }
-    KoInputDevice id;
-    KoToolManager::instance()->priv()->switchInputDevice(id);
-    if (d->activeTool == 0) {
-        event->ignore();
-        return;
-    }
-
     KoPointerEvent ev(event, point);
-    d->activeTool->mouseMoveEvent(&ev);
-
-    d->checkAutoScroll(ev);
+    mouseMoveEvent(&ev);
 }
 
 void KoToolProxy::mouseMoveEvent(KoPointerEvent *event)
@@ -364,38 +301,8 @@ void KoToolProxy::mouseMoveEvent(KoPointerEvent *event)
 
 void KoToolProxy::mouseReleaseEvent(QMouseEvent *event, const QPointF &point)
 {
-    d->mouseLeaveWorkaround = false;
-    KoInputDevice id;
-    KoToolManager::instance()->priv()->switchInputDevice(id);
-    d->scrollTimer.stop();
-
     KoPointerEvent ev(event, point);
-    if (d->activeTool) {
-        d->activeTool->mouseReleaseEvent(&ev);
-
-        if (! event->isAccepted() && event->button() == Qt::LeftButton && event->modifiers() == 0
-                && qAbs(d->mouseDownPoint.x() - event->x()) < 5
-                && qAbs(d->mouseDownPoint.y() - event->y()) < 5) {
-            // we potentially will change the selection
-            Q_ASSERT(d->activeTool->canvas());
-            KoShapeManager *manager = d->activeTool->canvas()->shapeManager();
-            Q_ASSERT(manager);
-            // only change the selection if that will not lead to losing a complex selection
-            if (manager->selection()->count() <= 1) {
-                KoShape *shape = manager->shapeAt(point);
-                if (shape && !manager->selection()->isSelected(shape)) { // make the clicked shape the active one
-                    manager->selection()->deselectAll();
-                    manager->selection()->select(shape);
-                    QList<KoShape*> shapes;
-                    shapes << shape;
-                    QString tool = KoToolManager::instance()->preferredToolForSelection(shapes);
-                    KoToolManager::instance()->switchToolRequested(tool);
-                }
-            }
-        }
-    } else {
-        event->ignore();
-    }
+    mouseReleaseEvent(&ev);
 }
 
 void KoToolProxy::mouseReleaseEvent(KoPointerEvent* event)
@@ -407,30 +314,11 @@ void KoToolProxy::mouseReleaseEvent(KoPointerEvent* event)
 
     if (d->activeTool) {
         d->activeTool->mouseReleaseEvent(event);
-
-        if (!event->isAccepted() && event->button() == Qt::LeftButton && event->modifiers() == 0
-                && qAbs(d->mouseDownPoint.x() - event->x()) < 5
-                && qAbs(d->mouseDownPoint.y() - event->y()) < 5) {
-            // we potentially will change the selection
-            Q_ASSERT(d->activeTool->canvas());
-            KoShapeManager *manager = d->activeTool->canvas()->shapeManager();
-            Q_ASSERT(manager);
-            // only change the selection if that will not lead to losing a complex selection
-            if (manager->selection()->count() <= 1) {
-                KoShape *shape = manager->shapeAt(event->point);
-                if (shape && !manager->selection()->isSelected(shape)) { // make the clicked shape the active one
-                    manager->selection()->deselectAll();
-                    manager->selection()->select(shape);
-                    QList<KoShape*> shapes;
-                    shapes << shape;
-                    QString tool = KoToolManager::instance()->preferredToolForSelection(shapes);
-                    KoToolManager::instance()->switchToolRequested(tool);
-                }
-            }
-        }
     } else {
         event->ignore();
     }
+
+    d->isToolPressed = false;
 }
 
 void KoToolProxy::keyPressEvent(QKeyEvent *event)
@@ -447,23 +335,15 @@ void KoToolProxy::keyReleaseEvent(QKeyEvent *event)
         d->activeTool->keyReleaseEvent(event);
     else
         event->ignore();
+
+    d->isToolPressed = false;
 }
 
-void KoToolProxy::wheelEvent(QWheelEvent *event, const QPointF &point)
+void KoToolProxy::explicitUserStrokeEndRequest()
 {
-    KoPointerEvent ev(event, point);
-    if (d->activeTool)
-        d->activeTool->wheelEvent(&ev);
-    else
-        event->ignore();
-}
-
-void KoToolProxy::wheelEvent(KoPointerEvent *event)
-{
-    if (d->activeTool)
-        d->activeTool->wheelEvent(event);
-    else
-        event->ignore();
+    if (d->activeTool) {
+        d->activeTool->explicitUserStrokeEndRequest();
+    }
 }
 
 QVariant KoToolProxy::inputMethodQuery(Qt::InputMethodQuery query, const KoViewConverter &converter) const
@@ -478,6 +358,11 @@ void KoToolProxy::inputMethodEvent(QInputMethodEvent *event)
     if (d->activeTool) d->activeTool->inputMethodEvent(event);
 }
 
+QMenu *KoToolProxy::popupActionsMenu()
+{
+    return d->activeTool ? d->activeTool->popupActionsMenu() : 0;
+}
+
 void KoToolProxy::setActiveTool(KoToolBase *tool)
 {
     if (d->activeTool)
@@ -490,16 +375,34 @@ void KoToolProxy::setActiveTool(KoToolBase *tool)
     }
 }
 
+void KoToolProxy::touchEvent(QTouchEvent* event, const QPointF& point)
+{
+    // only one "touchpoint" events should be here
+    KoPointerEvent ev(event, point);
+
+    if (!d->activeTool) return;
+
+    switch (event->touchPointStates())
+    {
+    case Qt::TouchPointPressed:
+        d->activeTool->mousePressEvent(&ev);
+        break;
+    case Qt::TouchPointMoved:
+        d->activeTool->mouseMoveEvent(&ev);
+        break;
+    case Qt::TouchPointReleased:
+        d->activeTool->mouseReleaseEvent(&ev);
+        break;
+    default: // don't care
+        ;
+    }
+}
+
 void KoToolProxyPrivate::setCanvasController(KoCanvasController *c)
 {
     controller = c;
 }
 
-QHash<QString, QAction *> KoToolProxy::actions() const
-{
-    return d->activeTool ? d->activeTool->actions() : QHash<QString, QAction *>();
-}
-
 bool KoToolProxy::hasSelection() const
 {
     return d->activeTool ? d->activeTool->hasSelection() : false;
@@ -520,72 +423,11 @@ void KoToolProxy::copy() const
 bool KoToolProxy::paste()
 {
     bool success = false;
-    KoCanvasBase *canvas = d->controller->canvas();
 
-    if (d->activeTool && d->isActiveLayerEditable())
+    if (d->activeTool && d->isActiveLayerEditable()) {
         success = d->activeTool->paste();
-
-    if (!success) {
-        const QMimeData *data = QApplication::clipboard()->mimeData();
-
-        if (data->hasFormat(KoOdf::mimeType(KoOdf::Text))) {
-            KoShapeManager *shapeManager = canvas->shapeManager();
-            KoShapePaste paste(canvas, shapeManager->selection()->activeLayer());
-            success = paste.paste(KoOdf::Text, data);
-            if (success) {
-                shapeManager->selection()->deselectAll();
-                Q_FOREACH (KoShape *shape, paste.pastedShapes()) {
-                    shapeManager->selection()->select(shape);
-                }
-            }
-        }
     }
 
-    if (!success) {
-        const QMimeData *data = QApplication::clipboard()->mimeData();
-
-        QList<QImage> imageList;
-
-        QImage image = QApplication::clipboard()->image();
-
-        if (!image.isNull()) {
-            imageList << image;
-        }
-        // QT5TODO: figure out how to download data synchronously, which is deprecated in frameworks.
-        else if (data->hasUrls()) {
-            QList<QUrl> urls = QApplication::clipboard()->mimeData()->urls();
-            foreach (const QUrl &url, urls) {
-                QImage image;
-                image.load(url.toLocalFile());
-                if (!image.isNull()) {
-                    imageList << image;
-                }
-            }
-        }
-
-        KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value("PictureShape");
-        QWidget *canvasWidget = canvas->canvasWidget();
-        const KoViewConverter *converter = canvas->viewConverter();
-        if (imageList.length() > 0 && factory && canvasWidget) {
-            KUndo2Command *cmd = new KUndo2Command(kundo2_i18n("Paste Image"));
-            Q_FOREACH (const QImage &image, imageList) {
-                if (!image.isNull()) {
-                    QPointF p = converter->viewToDocument(canvasWidget->mapFromGlobal(QCursor::pos()) + canvas->canvasController()->documentOffset()- canvasWidget->pos());
-                    KoProperties params;
-                    params.setProperty("qimage", image);
-
-                    KoShape *shape = factory->createShape(&params, canvas->shapeController()->resourceManager());
-                    shape->setPosition(p);
-
-                    // add shape to the document
-                    canvas->shapeController()->addShapeDirect(shape, cmd);
-
-                    success = true;
-                }
-            }
-            canvas->addCommand(cmd);
-        }
-    }
     return success;
 }
 
@@ -607,38 +449,44 @@ void KoToolProxy::dropEvent(QDropEvent *event, const QPointF &point)
         d->activeTool->dropEvent(event, point);
 }
 
-QStringList KoToolProxy::supportedPasteMimeTypes() const
-{
-    if (d->activeTool)
-        return d->activeTool->supportedPasteMimeTypes();
-
-    return QStringList();
-}
-
-QList<QAction*> KoToolProxy::popupActionList() const
-{
-    if (d->activeTool)
-        return d->activeTool->popupActionList();
-    return QList<QAction*>();
-}
-
 void KoToolProxy::deleteSelection()
 {
     if (d->activeTool)
-        return d->activeTool->deleteSelection();
+        d->activeTool->deleteSelection();
 }
 
 void KoToolProxy::processEvent(QEvent *e) const
 {
     if(e->type()==QEvent::ShortcutOverride
-       && d->activeTool
-       && d->activeTool->isInTextMode()
-       && (static_cast<QKeyEvent*>(e)->modifiers()==Qt::NoModifier ||
-           static_cast<QKeyEvent*>(e)->modifiers()==Qt::ShiftModifier)) {
+            && d->activeTool
+            && d->activeTool->isInTextMode()
+            && (static_cast<QKeyEvent*>(e)->modifiers()==Qt::NoModifier ||
+                static_cast<QKeyEvent*>(e)->modifiers()==Qt::ShiftModifier)) {
         e->accept();
     }
 }
 
+void KoToolProxy::requestUndoDuringStroke()
+{
+    if (d->activeTool) {
+        d->activeTool->requestUndoDuringStroke();
+    }
+}
+
+void KoToolProxy::requestStrokeCancellation()
+{
+    if (d->activeTool) {
+        d->activeTool->requestStrokeCancellation();
+    }
+}
+
+void KoToolProxy::requestStrokeEnd()
+{
+    if (d->activeTool) {
+        d->activeTool->requestStrokeEnd();
+    }
+}
+
 KoToolProxyPrivate *KoToolProxy::priv()
 {
     return d;
