@@ -241,7 +241,7 @@ qreal KisMaskGenerator::effectiveSrcHeight() const
      * don't take spikes into account, they will be generated from
      * this data.
      */
-    return d->diameter * d->ratio * d->scaleX;
+    return d->diameter * d->ratio * d->scaleY;
 }
 
 qreal KisMaskGenerator::diameter() const
@@ -249,6 +249,13 @@ qreal KisMaskGenerator::diameter() const
     return d->diameter;
 }
 
+void KisMaskGenerator::setDiameter(qreal value)
+{
+    d->diameter = value;
+    init();
+    setScale(d->scaleX, d->scaleY);
+}
+
 qreal KisMaskGenerator::ratio() const
 {
     return d->ratio;
