@@ -35,15 +35,18 @@
 #include "kis_global.h"
 #include <cmath>
 
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
 #include <errno.h>
 #endif
 
 KisImageConfig::KisImageConfig(bool readOnly)
-    : m_config( KSharedConfig::openConfig()->group(QString())),
-      m_readOnly(readOnly)
+    : m_config(KSharedConfig::openConfig()->group(QString()))
+    , m_readOnly(readOnly)
 {
-#ifdef Q_OS_OSX
+    if (!readOnly) {
+        KIS_SAFE_ASSERT_RECOVER_RETURN(qApp->thread() == QThread::currentThread());
+    }
+#ifdef Q_OS_MACOS
     // clear /var/folders/ swap path set by old broken Krita swap implementation in order to use new default swap dir.
     QString swap = m_config.readEntry("swaplocation", "");
     if (swap.startsWith("/var/folders/")) {
@@ -228,7 +231,7 @@ void KisImageConfig::setMemoryPoolLimitPercent(qreal value)
 
 QString KisImageConfig::safelyGetWritableTempLocation(const QString &suffix, const QString &configKey, bool requestDefault) const
 {
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
     // On OSX, QDir::tempPath() gives us a folder we cannot reply upon (usually
     // something like /var/folders/.../...) and that will have vanished when we
     // try to create the tmp file in KisMemoryWindow::KisMemoryWindow using
@@ -356,7 +359,7 @@ void KisImageConfig::setLazyFrameCreationEnabled(bool value)
 #include <sys/sysctl.h>
 #elif defined Q_OS_WIN
 #include <windows.h>
-#elif defined Q_OS_OSX
+#elif defined Q_OS_MACOS
 #include <sys/types.h>
 #include <sys/sysctl.h>
 #endif
@@ -400,7 +403,7 @@ int KisImageConfig::totalRAM()
 #   if defined ENV32BIT
     totalMemory = qMin(totalMemory, 2000);
 #   endif
-#elif defined Q_OS_OSX
+#elif defined Q_OS_MACOS
     int mib[2] = { CTL_HW, HW_MEMSIZE };
     u_int namelen = sizeof(mib) / sizeof(mib[0]);
     uint64_t size;
@@ -527,11 +530,7 @@ void KisImageConfig::setFpsLimit(int value)
 
 bool KisImageConfig::useOnDiskAnimationCacheSwapping(bool defaultValue) const
 {
-#ifdef Q_OS_WIN
-    return false;
-#else
     return defaultValue ? true : m_config.readEntry("useOnDiskAnimationCacheSwapping", true);
-#endif
 }
 
 void KisImageConfig::setUseOnDiskAnimationCacheSwapping(bool value)
@@ -588,3 +587,14 @@ void KisImageConfig::setAnimationCacheRegionOfInterestMargin(qreal value)
 {
     m_config.writeEntry("animationCacheRegionOfInterestMargin", value);
 }
+
+QColor KisImageConfig::selectionOverlayMaskColor(bool defaultValue) const
+{
+    QColor def(255, 0, 0, 128);
+    return (defaultValue ? def : m_config.readEntry("selectionOverlayMaskColor", def));
+}
+
+void KisImageConfig::setSelectionOverlayMaskColor(const QColor &color)
+{
+    m_config.writeEntry("selectionOverlayMaskColor", color);
+}
