@@ -1,28 +1,30 @@
-/* This file is part of the KDE project
-   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+/*
+ * Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
+ * Copyright (C) 2012 Boudewijn Rempt <boud@valdyas.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
-*/
+ */
 
 #ifndef KIS_APPLICATION_H
 #define KIS_APPLICATION_H
 
+#include <QPointer>
+#include <QScopedPointer>
 #include <qtsingleapplication/qtsingleapplication.h>
 #include "kritaui_export.h"
-#include <KisAutoSaveRecoveryDialog.h>
 
 class KisMainWindow;
 class KisApplicationPrivate;
@@ -58,7 +60,7 @@ public:
     /**
      *  Destructor.
      */
-    virtual ~KisApplication();
+    ~KisApplication() override;
 
     /**
      * Call this to start the application.
@@ -83,40 +85,40 @@ public:
     /**
      * Tell KisApplication to show this splashscreen when you call start();
      * when start returns, the splashscreen is hidden. Use KSplashScreen
-     * to have the splash show correctly on Xinerama displays. 
+     * to have the splash show correctly on Xinerama displays.
      */
     void setSplashScreen(QWidget *splash);
 
     void setSplashScreenLoadingText(QString);
 
-
-
+    void hideSplashScreen();
 
     /// Overridden to handle exceptions from event handlers.
-    bool notify(QObject *receiver, QEvent *event);
+    bool notify(QObject *receiver, QEvent *event) override;
+
+    void addResourceTypes();
+    void loadResources();
+    void loadResourceTags();
+    void loadPlugins();
+    void loadGuiPlugins();
+    void initializeGlobals(const KisApplicationArguments &args);
 
 public Q_SLOTS:
 
     void remoteArguments(QByteArray message, QObject*socket);
     void fileOpenRequested(const QString & url);
 
-    void onAutoSaveFinished(int result);
-
 private:
     /// @return the number of autosavefiles opened
     void checkAutosaveFiles();
-    bool createNewDocFromTemplate(const QString &fileName, KisMainWindow *mainWindow);
+    bool createNewDocFromTemplate(const QString &fileName, KisMainWindow *m_mainWindow);
     void clearConfig();
-    void loadResources();
-    void loadPlugins();
 
 private:
-    KisApplicationPrivate * const d;
+    class Private;
+    QScopedPointer<Private> d;
     class ResetStarting;
     friend class ResetStarting;
-    KisAutoSaveRecoveryDialog *dlg;
-    QStringList autoSaveFiles;
-    KisMainWindow *mainWindow;
 };
 
 #endif
