@@ -39,6 +39,8 @@
 #include "KisViewManager.h"
 #include "kis_selection_manager.h"
 #include "KisDocument.h"
+#include "kis_update_info.h"
+
 
 struct KisCanvasWidgetBase::Private
 {
@@ -52,7 +54,7 @@ public:
         , borderColor(Qt::gray)
     {}
 
-    QList<QPointer<KisCanvasDecoration>> decorations;
+    QList<KisCanvasDecorationSP> decorations;
     KisCanvas2 * canvas;
     KisCoordinatesConverter *coordinatesConverter;
     const KoViewConverter * viewConverter;
@@ -72,11 +74,11 @@ KisCanvasWidgetBase::KisCanvasWidgetBase(KisCanvas2 * canvas, KisCoordinatesConv
 KisCanvasWidgetBase::~KisCanvasWidgetBase()
 {
     /**
-     * Clear all the attached decoration. Oherwise they might decide
+     * Clear all the attached decoration. Otherwise they might decide
      * to process some events or signals after the canvas has been
      * destroyed
      */
-    qDeleteAll(m_d->decorations);
+    //5qDeleteAll(m_d->decorations);
     m_d->decorations.clear();
 
     delete m_d;
@@ -84,16 +86,18 @@ KisCanvasWidgetBase::~KisCanvasWidgetBase()
 
 void KisCanvasWidgetBase::drawDecorations(QPainter & gc, const QRect &updateWidgetRect) const
 {
-    gc.save();
     if (!m_d->canvas) {
         dbgFile<<"canvas doesn't exist, in canvas widget base!";
+        return;
     }
+    gc.save();
+
     // Setup the painter to take care of the offset; all that the
     // classes that do painting need to keep track of is resolution
     gc.setRenderHint(QPainter::Antialiasing);
     gc.setRenderHint(QPainter::TextAntialiasing);
 
-    // This option does not do anything anymore with Qt4.6, so don't reenable it since it seems to break display
+    // This option does not do anything anymore with Qt4.6, so don't re-enable it since it seems to break display
     // http://www.archivum.info/qt-interest@trolltech.com/2010-01/00481/Re:-(Qt-interest)-Is-QPainter::HighQualityAntialiasing-render-hint-broken-in-Qt-4.6.html
     // gc.setRenderHint(QPainter::HighQualityAntialiasing);
 
@@ -141,28 +145,41 @@ void KisCanvasWidgetBase::drawDecorations(QPainter & gc, const QRect &updateWidg
         gc.restore();
     }
 
+    gc.restore();
+
+    // ask the decorations to paint themselves
+    Q_FOREACH (KisCanvasDecorationSP deco, m_d->decorations) {
+        deco->paint(gc, m_d->coordinatesConverter->widgetToDocument(updateWidgetRect), m_d->coordinatesConverter,m_d->canvas);
+    }
+
+    gc.setTransform(transform);
     // - some tools do not restore gc, but that is not important here
     // - we need to disable clipping to draw handles properly
     gc.setClipping(false);
     toolProxy()->paint(gc, *m_d->viewConverter);
-    gc.restore();
-
-    // ask the decorations to paint themselves
-    Q_FOREACH (KisCanvasDecoration* deco, m_d->decorations) {
-        deco->paint(gc, m_d->coordinatesConverter->widgetToDocument(updateWidgetRect), m_d->coordinatesConverter,m_d->canvas);
-    }
 
     gc.restore();
 }
 
-void KisCanvasWidgetBase::addDecoration(KisCanvasDecoration* deco)
+void KisCanvasWidgetBase::addDecoration(KisCanvasDecorationSP deco)
 {
     m_d->decorations.push_back(deco);
+    std::stable_sort(m_d->decorations.begin(), m_d->decorations.end(), KisCanvasDecoration::comparePriority);
 }
 
-KisCanvasDecoration* KisCanvasWidgetBase::decoration(const QString& id) const
+void KisCanvasWidgetBase::removeDecoration(const QString &id)
 {
-    Q_FOREACH (QPointer<KisCanvasDecoration> deco, m_d->decorations) {
+    for (auto it = m_d->decorations.begin(); it != m_d->decorations.end(); ++it) {
+        if ((*it)->id() == id) {
+            it = m_d->decorations.erase(it);
+            break;
+        }
+    }
+}
+
+KisCanvasDecorationSP KisCanvasWidgetBase::decoration(const QString& id) const
+{
+    Q_FOREACH (KisCanvasDecorationSP deco, m_d->decorations) {
         if (deco->id() == id) {
             return deco;
         }
@@ -170,12 +187,13 @@ KisCanvasDecoration* KisCanvasWidgetBase::decoration(const QString& id) const
     return 0;
 }
 
-void KisCanvasWidgetBase::setDecorations(const QList<QPointer<KisCanvasDecoration> > &decorations)
+void KisCanvasWidgetBase::setDecorations(const QList<KisCanvasDecorationSP > &decorations)
 {
     m_d->decorations=decorations;
+    std::stable_sort(m_d->decorations.begin(), m_d->decorations.end(), KisCanvasDecoration::comparePriority);
 }
 
-QList<QPointer<KisCanvasDecoration> > KisCanvasWidgetBase::decorations() const
+QList<KisCanvasDecorationSP > KisCanvasWidgetBase::decorations() const
 {
     return m_d->decorations;
 }
@@ -187,7 +205,7 @@ void KisCanvasWidgetBase::setWrapAroundViewingMode(bool value)
 
 QImage KisCanvasWidgetBase::createCheckersImage(qint32 checkSize)
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
 
     if(checkSize < 0)
         checkSize = cfg.checkSize();
@@ -207,7 +225,7 @@ QImage KisCanvasWidgetBase::createCheckersImage(qint32 checkSize)
 
 void KisCanvasWidgetBase::notifyConfigChanged()
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
     m_d->borderColor = cfg.canvasBorderColor();
 }
 
@@ -226,15 +244,22 @@ KisCoordinatesConverter* KisCanvasWidgetBase::coordinatesConverter() const
     return m_d->coordinatesConverter;
 }
 
+QVector<QRect> KisCanvasWidgetBase::updateCanvasProjection(const QVector<KisUpdateInfoSP> &infoObjects)
+{
+    QVector<QRect> dirtyViewRects;
+
+    Q_FOREACH (KisUpdateInfoSP info, infoObjects) {
+        dirtyViewRects << this->updateCanvasProjection(info);
+    }
+
+    return dirtyViewRects;
+}
+
 KoToolProxy *KisCanvasWidgetBase::toolProxy() const
 {
     return m_d->toolProxy;
 }
 
-void KisCanvasWidgetBase::setDisplayFilter(KisDisplayFilter */*displayFilter*/)
-{
-}
-
 QVariant KisCanvasWidgetBase::processInputMethodQuery(Qt::InputMethodQuery query) const
 {
     if (query == Qt::ImMicroFocus) {
