@@ -62,7 +62,6 @@
 #include "kis_config.h"
 #include "kis_config_notifier.h"
 #include "kis_cursor.h"
-#include <recorder/kis_recorded_paint_action.h>
 #include <kis_selection_mask.h>
 #include "kis_resources_snapshot.h"
 #include <KisView.h>
@@ -70,7 +69,6 @@
 #include "kis_tool_utils.h"
 
 
-
 struct Q_DECL_HIDDEN KisTool::Private {
     QCursor cursor; // the cursor that should be shown on tool activation.
 
@@ -80,7 +78,7 @@ struct Q_DECL_HIDDEN KisTool::Private {
     KoColor currentFgColor;
     KoColor currentBgColor;
     float currentExposure{1.0};
-    KisFilterConfiguration* currentGenerator{0};
+    KisFilterConfigurationSP currentGenerator;
     QWidget* optionWidget{0};
     ToolMode m_mode{HOVER_MODE};
     bool m_isActive{false};
@@ -93,23 +91,7 @@ KisTool::KisTool(KoCanvasBase * canvas, const QCursor & cursor)
     d->cursor = cursor;
 
     connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(resetCursorStyle()));
-    connect(this, SIGNAL(isActiveChanged()), SLOT(resetCursorStyle()));
-
-    KActionCollection *collection = this->canvas()->canvasController()->actionCollection();
-
-    if (!collection->action("toggle_fg_bg")) {
-        QAction *toggleFgBg = KisActionRegistry::instance()->makeQAction("toggle_fg_bg", collection);
-        collection->addAction("toggle_fg_bg", toggleFgBg);
-    }
-
-    if (!collection->action("reset_fg_bg")) {
-        QAction *toggleFgBg = KisActionRegistry::instance()->makeQAction("reset_fg_bg", collection);
-        collection->addAction("reset_fg_bg", toggleFgBg);
-    }
-
-    addAction("toggle_fg_bg", dynamic_cast<QAction *>(collection->action("toggle_fg_bg")));
-    addAction("reset_fg_bg", dynamic_cast<QAction *>(collection->action("reset_fg_bg")));
-
+    connect(this, SIGNAL(isActiveChanged(bool)), SLOT(resetCursorStyle()));
 }
 
 KisTool::~KisTool()
@@ -117,10 +99,9 @@ KisTool::~KisTool()
     delete d;
 }
 
-void KisTool::activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes)
+void KisTool::activate(ToolActivation activation, const QSet<KoShape*> &shapes)
 {
-    Q_UNUSED(toolActivation);
-    Q_UNUSED(shapes);
+    KoToolBase::activate(activation, shapes);
 
     resetCursorStyle();
 
@@ -128,8 +109,8 @@ void KisTool::activate(ToolActivation toolActivation, const QSet<KoShape*> &shap
     if (!canvas()->resourceManager()) return;
 
 
-    d->currentFgColor = canvas()->resourceManager()->resource(KoCanvasResourceManager::ForegroundColor).value<KoColor>();
-    d->currentBgColor = canvas()->resourceManager()->resource(KoCanvasResourceManager::BackgroundColor).value<KoColor>();
+    d->currentFgColor = canvas()->resourceManager()->resource(KoCanvasResourceProvider::ForegroundColor).value<KoColor>();
+    d->currentBgColor = canvas()->resourceManager()->resource(KoCanvasResourceProvider::BackgroundColor).value<KoColor>();
 
     if (canvas()->resourceManager()->hasResource(KisCanvasResourceProvider::CurrentPattern)) {
         d->currentPattern = canvas()->resourceManager()->resource(KisCanvasResourceProvider::CurrentPattern).value<KoPattern*>();
@@ -152,62 +133,34 @@ void KisTool::activate(ToolActivation toolActivation, const QSet<KoShape*> &shap
         d->currentGenerator = canvas()->resourceManager()->resource(KisCanvasResourceProvider::CurrentGeneratorConfiguration).value<KisFilterConfiguration*>();
     }
 
-    connect(actions().value("toggle_fg_bg"), SIGNAL(triggered()), SLOT(slotToggleFgBg()), Qt::UniqueConnection);
-    connect(actions().value("reset_fg_bg"), SIGNAL(triggered()), SLOT(slotResetFgBg()), Qt::UniqueConnection);
-    connect(image(), SIGNAL(sigUndoDuringStrokeRequested()), SLOT(requestUndoDuringStroke()), Qt::UniqueConnection);
-    connect(image(), SIGNAL(sigStrokeCancellationRequested()), SLOT(requestStrokeCancellation()), Qt::UniqueConnection);
-    connect(image(), SIGNAL(sigStrokeEndRequested()), SLOT(requestStrokeEnd()), Qt::UniqueConnection);
-
     d->m_isActive = true;
-    emit isActiveChanged();
+    emit isActiveChanged(true);
 }
 
 void KisTool::deactivate()
 {
-    bool result = true;
-
-    result &= disconnect(image().data(), SIGNAL(sigUndoDuringStrokeRequested()), this, 0);
-    result &= disconnect(image().data(), SIGNAL(sigStrokeCancellationRequested()), this, 0);
-    result &= disconnect(image().data(), SIGNAL(sigStrokeEndRequested()), this, 0);
-    result &= disconnect(actions().value("toggle_fg_bg"), 0, this, 0);
-    result &= disconnect(actions().value("reset_fg_bg"), 0, this, 0);
-
-    if (!result) {
-        warnKrita << "WARNING: KisTool::deactivate() failed to disconnect"
-                   << "some signal connections. Your actions might be executed twice!";
-    }
-
     d->m_isActive = false;
-    emit isActiveChanged();
-}
+    emit isActiveChanged(false);
 
-void KisTool::requestUndoDuringStroke()
-{
-    /**
-     * Default implementation just cancells the stroke
-     */
-    requestStrokeCancellation();
-}
-
-void KisTool::requestStrokeCancellation()
-{
-}
-
-void KisTool::requestStrokeEnd()
-{
+    KoToolBase::deactivate();
 }
 
 void KisTool::canvasResourceChanged(int key, const QVariant & v)
 {
+    QString formattedBrushName;
+    if (key == KisCanvasResourceProvider::CurrentPaintOpPreset) {
+         formattedBrushName = v.value<KisPaintOpPresetSP>()->name().replace("_", " ");
+    }
+
     switch (key) {
-    case(KoCanvasResourceManager::ForegroundColor):
+    case(KoCanvasResourceProvider::ForegroundColor):
         d->currentFgColor = v.value<KoColor>();
         break;
-    case(KoCanvasResourceManager::BackgroundColor):
+    case(KoCanvasResourceProvider::BackgroundColor):
         d->currentBgColor = v.value<KoColor>();
         break;
     case(KisCanvasResourceProvider::CurrentPattern):
-        d->currentPattern = static_cast<KoPattern *>(v.value<void *>());
+        d->currentPattern = v.value<KoPattern*>();
         break;
     case(KisCanvasResourceProvider::CurrentGradient):
         d->currentGradient = static_cast<KoAbstractGradient *>(v.value<void *>());
@@ -219,7 +172,7 @@ void KisTool::canvasResourceChanged(int key, const QVariant & v)
         d->currentGenerator = static_cast<KisFilterConfiguration*>(v.value<void *>());
         break;
     case(KisCanvasResourceProvider::CurrentPaintOpPreset):
-        emit statusTextChanged(v.value<KisPaintOpPresetSP>()->name());
+        emit statusTextChanged(formattedBrushName);
         break;
     case(KisCanvasResourceProvider::CurrentKritaNode):
         resetCursorStyle();
@@ -288,12 +241,12 @@ QPointF KisTool::convertToPixelCoordAndSnap(const QPointF& pt, const QPointF &of
     return image()->documentToPixel(pos);
 }
 
-QPoint KisTool::convertToIntPixelCoord(KoPointerEvent *e)
+QPoint KisTool::convertToImagePixelCoordFloored(KoPointerEvent *e)
 {
     if (!image())
         return e->point.toPoint();
 
-    return image()->documentToIntPixel(e->point);
+    return image()->documentToImagePixelFloored(e->point);
 }
 
 QPointF KisTool::viewToPixel(const QPointF &viewCoord) const
@@ -311,10 +264,16 @@ QRectF KisTool::convertToPt(const QRectF &rect)
     QRectF r;
     //We add 1 in the following to the extreme coords because a pixel always has size
     r.setCoords(int(rect.left()) / image()->xRes(), int(rect.top()) / image()->yRes(),
-                int(1 + rect.right()) / image()->xRes(), int(1 + rect.bottom()) / image()->yRes());
+                int(rect.right()) / image()->xRes(), int( rect.bottom()) / image()->yRes());
     return r;
 }
 
+qreal KisTool::convertToPt(qreal value)
+{
+    const qreal avgResolution = 0.5 * (image()->xRes() + image()->yRes());
+    return value / avgResolution;
+}
+
 QPointF KisTool::pixelToView(const QPoint &pixelCoord) const
 {
     if (!image())
@@ -407,7 +366,7 @@ KisPaintOpPresetSP KisTool::currentPaintOpPreset()
     return canvas()->resourceManager()->resource(KisCanvasResourceProvider::CurrentPaintOpPreset).value<KisPaintOpPresetSP>();
 }
 
-KisNodeSP KisTool::currentNode()
+KisNodeSP KisTool::currentNode() const
 {
     KisNodeSP node = canvas()->resourceManager()->resource(KisCanvasResourceProvider::CurrentKritaNode).value<KisNodeWSP>();
     return node;
@@ -435,7 +394,7 @@ KisImageWSP KisTool::currentImage()
     return image();
 }
 
-KisFilterConfiguration * KisTool::currentGenerator()
+KisFilterConfigurationSP  KisTool::currentGenerator()
 {
     return d->currentGenerator;
 }
@@ -448,6 +407,11 @@ KisTool::ToolMode KisTool::mode() const {
     return d->m_mode;
 }
 
+void KisTool::setCursor(const QCursor &cursor)
+{
+    d->cursor = cursor;
+}
+
 KisTool::AlternateAction KisTool::actionToAlternateAction(ToolAction action) {
     KIS_ASSERT_RECOVER_RETURN_VALUE(action != Primary, Secondary);
     return (AlternateAction)action;
@@ -549,17 +513,34 @@ void KisTool::mouseMoveEvent(KoPointerEvent *event)
 void KisTool::deleteSelection()
 {
     KisResourcesSnapshotSP resources =
-        new KisResourcesSnapshot(image(), currentNode(), 0, this->canvas()->resourceManager());
+        new KisResourcesSnapshot(image(), currentNode(), this->canvas()->resourceManager());
+
+    if (!blockUntilOperationsFinished()) {
+        return;
+    }
 
     if (!KisToolUtils::clearImage(image(), resources->currentNode(), resources->activeSelection())) {
         KoToolBase::deleteSelection();
     }
 }
 
-void KisTool::setupPaintAction(KisRecordedPaintAction* action)
+KisTool::NodePaintAbility KisTool::nodePaintAbility()
 {
-    action->setPaintColor(currentFgColor());
-    action->setBackgroundColor(currentBgColor());
+    KisNodeSP node = currentNode();
+    if (!node) {
+        return NodePaintAbility::UNPAINTABLE;
+    }
+    if (node->inherits("KisShapeLayer")) {
+        return NodePaintAbility::VECTOR;
+    }
+    if (node->inherits("KisCloneLayer")) {
+        return NodePaintAbility::CLONE;
+    }
+    if (node->paintDevice()) {
+        return NodePaintAbility::PAINT;
+    }
+
+    return NodePaintAbility::UNPAINTABLE;
 }
 
 QWidget* KisTool::createOptionWidget()
@@ -582,9 +563,11 @@ void KisTool::paintToolOutline(QPainter* painter, const QPainterPath &path)
         painter->endNativePainting();
     }
     else {
+        painter->save();
         painter->setCompositionMode(QPainter::RasterOp_SourceXorDestination);
         painter->setPen(QColor(128, 255, 128));
         painter->drawPath(path);
+        painter->restore();
     }
 }
 
@@ -607,51 +590,48 @@ bool KisTool::overrideCursorIfNotEditable()
     return false;
 }
 
+bool KisTool::blockUntilOperationsFinished()
+{
+    KisCanvas2 * kiscanvas = static_cast<KisCanvas2*>(canvas());
+    KisViewManager* viewManager = kiscanvas->viewManager();
+    return viewManager->blockUntilOperationsFinished(image());
+}
+
+void KisTool::blockUntilOperationsFinishedForced()
+{
+    KisCanvas2 * kiscanvas = static_cast<KisCanvas2*>(canvas());
+    KisViewManager* viewManager = kiscanvas->viewManager();
+    viewManager->blockUntilOperationsFinishedForced(image());
+}
+
 bool KisTool::isActive() const
 {
     return d->m_isActive;
 }
 
-void KisTool::slotToggleFgBg()
-{
-    KoCanvasResourceManager* resourceManager = canvas()->resourceManager();
-    KoColor newFg = resourceManager->backgroundColor();
-    KoColor newBg = resourceManager->foregroundColor();
-
-    /**
-     * NOTE: Some of color selectors do not differentiate foreground
-     *       and background colors, so if one wants them to end up
-     *       being set up to foreground color, it should be set the
-     *       last.
-     */
-    resourceManager->setBackgroundColor(newBg);
-    resourceManager->setForegroundColor(newFg);
-}
-
-void KisTool::slotResetFgBg()
-{
-    KoCanvasResourceManager* resourceManager = canvas()->resourceManager();
-
-    // see a comment in slotToggleFgBg()
-    resourceManager->setBackgroundColor(KoColor(Qt::white, KoColorSpaceRegistry::instance()->rgb8()));
-    resourceManager->setForegroundColor(KoColor(Qt::black, KoColorSpaceRegistry::instance()->rgb8()));
-}
-
-
-void KisTool::setCurrentNodeLocked(bool locked)
-{
-    if (currentNode()) {
-        currentNode()->setSystemLocked(locked, false);
-    }
-}
-
 bool KisTool::nodeEditable()
 {
     KisNodeSP node = currentNode();
     if (!node) {
         return false;
     }
-    if (!node->isEditable()) {
+
+    bool blockedNoIndirectPainting = false;
+
+    const bool presetUsesIndirectPainting =
+        !currentPaintOpPreset()->settings()->paintIncremental();
+
+    if (!presetUsesIndirectPainting) {
+        const KisIndirectPaintingSupport *indirectPaintingLayer =
+                dynamic_cast<const KisIndirectPaintingSupport*>(node.data());
+        if (indirectPaintingLayer) {
+            blockedNoIndirectPainting = !indirectPaintingLayer->supportsNonIndirectPainting();
+        }
+    }
+
+    bool nodeEditable = node->isEditable() && !blockedNoIndirectPainting;
+
+    if (!nodeEditable) {
         KisCanvas2 * kiscanvas = static_cast<KisCanvas2*>(canvas());
         QString message;
         if (!node->visible() && node->userLocked()) {
@@ -660,12 +640,14 @@ bool KisTool::nodeEditable()
             message = i18n("Layer is locked.");
         } else if(!node->visible()) {
             message = i18n("Layer is invisible.");
+        } else if (blockedNoIndirectPainting) {
+            message = i18n("Layer can be painted in Wash Mode only.");
         } else {
             message = i18n("Group not editable.");
         }
         kiscanvas->viewManager()->showFloatingMessage(message, KisIconUtils::loadIcon("object-locked"));
     }
-    return node->isEditable();
+    return nodeEditable;
 }
 
 bool KisTool::selectionEditable()
