@@ -43,7 +43,7 @@
 
 #include <kis_icon.h>
 #include <KoShape.h>
-#include <KoCanvasResourceManager.h>
+#include <KoCanvasResourceProvider.h>
 #include <KoColorSpace.h>
 #include <KoPointerEvent.h>
 #include <KoColor.h>
@@ -91,7 +91,7 @@ KisToolPaint::KisToolPaint(KoCanvasBase *canvas, const QCursor &cursor)
     m_supportOutline = false;
 
     {
-        int maxSize = KisConfig().readEntry("maximumBrushSize", 1000);
+        int maxSize = KisConfig(true).readEntry("maximumBrushSize", 1000);
 
         int brushSize = 1;
         do {
@@ -104,25 +104,8 @@ KisToolPaint::KisToolPaint(KoCanvasBase *canvas, const QCursor &cursor)
     }
 
     KisCanvas2 *kiscanvas = dynamic_cast<KisCanvas2*>(canvas);
-    KisActionManager *actionManager = kiscanvas->viewManager()->actionManager();
 
-    // XXX: Perhaps a better place for these?
-    if (!actionManager->actionByName("increase_brush_size")) {
-        KisAction *increaseBrushSize = new KisAction(i18n("Increase Brush Size"));
-        increaseBrushSize->setShortcut(Qt::Key_BracketRight);
-        actionManager->addAction("increase_brush_size", increaseBrushSize);
-    }
-
-    if (!actionManager->actionByName("decrease_brush_size")) {
-        KisAction *decreaseBrushSize = new KisAction(i18n("Decrease Brush Size"));
-        decreaseBrushSize->setShortcut(Qt::Key_BracketLeft);
-        actionManager->addAction("decrease_brush_size", decreaseBrushSize);
-    }
-
-    addAction("increase_brush_size", dynamic_cast<QAction *>(actionManager->actionByName("increase_brush_size")));
-    addAction("decrease_brush_size", dynamic_cast<QAction *>(actionManager->actionByName("decrease_brush_size")));
-
-    connect(this, SIGNAL(sigPaintingFinished()), kiscanvas->viewManager()->resourceProvider(), SLOT(slotPainting()));
+    connect(this, SIGNAL(sigPaintingFinished()), kiscanvas->viewManager()->canvasResourceProvider(), SLOT(slotPainting()));
 
     m_colorPickerDelayTimer.setSingleShot(true);
     connect(&m_colorPickerDelayTimer, SIGNAL(timeout()), this, SLOT(activatePickColorDelayed()));
@@ -174,7 +157,7 @@ void KisToolPaint::activate(ToolActivation toolActivation, const QSet<KoShape*>
         connect(action("decrease_brush_size"), SIGNAL(triggered()), SLOT(decreaseBrushSize()), Qt::UniqueConnection);
     }
 
-    KisCanvasResourceProvider *provider = qobject_cast<KisCanvas2*>(canvas())->viewManager()->resourceProvider();
+    KisCanvasResourceProvider *provider = qobject_cast<KisCanvas2*>(canvas())->viewManager()->canvasResourceProvider();
     m_oldOpacity = provider->opacity();
     provider->setOpacity(m_localOpacity);
 }
@@ -186,7 +169,7 @@ void KisToolPaint::deactivate()
         disconnect(action("decrease_brush_size"), 0, this, 0);
     }
 
-    KisCanvasResourceProvider *provider = qobject_cast<KisCanvas2*>(canvas())->viewManager()->resourceProvider();
+    KisCanvasResourceProvider *provider = qobject_cast<KisCanvas2*>(canvas())->viewManager()->canvasResourceProvider();
     m_localOpacity = provider->opacity();
     provider->setOpacity(m_oldOpacity);
 
@@ -195,7 +178,7 @@ void KisToolPaint::deactivate()
 
 QPainterPath KisToolPaint::tryFixBrushOutline(const QPainterPath &originalOutline)
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
     if (cfg.newOutlineStyle() == OUTLINE_NONE) return originalOutline;
 
     const qreal minThresholdSize = cfg.outlineSizeMinimum();
@@ -300,15 +283,15 @@ void KisToolPaint::activateAlternateAction(AlternateAction action)
 {
     switch (action) {
     case PickFgNode:
-        /* Falls through */
+        Q_FALLTHROUGH();
     case PickBgNode:
-        /* Falls through */
+        Q_FALLTHROUGH();
     case PickFgImage:
-        /* Falls through */
+        Q_FALLTHROUGH();
     case PickBgImage:
         delayedAction = action;
         m_colorPickerDelayTimer.start(100);
-        /* Falls through */
+        Q_FALLTHROUGH();
     default:
         pickColorWasOverridden();
         KisTool::activateAlternateAction(action);
@@ -448,7 +431,7 @@ int KisToolPaint::colorPreviewResourceId(AlternateAction action)
 {
     bool toForegroundColor = action == PickFgNode || action == PickFgImage;
     int resource = toForegroundColor ?
-        KoCanvasResourceManager::ForegroundColor : KoCanvasResourceManager::BackgroundColor;
+        KoCanvasResourceProvider::ForegroundColor : KoCanvasResourceProvider::BackgroundColor;
 
     return resource;
 }
@@ -614,24 +597,6 @@ void KisToolPaint::slotPopupQuickHelp()
     QWhatsThis::showText(QCursor::pos(), quickHelp());
 }
 
-KisToolPaint::NodePaintAbility KisToolPaint::nodePaintAbility()
-{
-    KisNodeSP node = currentNode();
-    if (!node) {
-        return NONE;
-    }
-    if (node->inherits("KisShapeLayer")) {
-        return VECTOR;
-    }
-    if (node->inherits("KisCloneLayer")) {
-        return CLONE;
-    }
-    if (node->paintDevice()) {
-        return PAINT;
-    }
-    return NONE;
-}
-
 void KisToolPaint::activatePrimaryAction()
 {
     pickColorWasOverridden();
@@ -691,7 +656,7 @@ QRectF KisToolPaint::colorPreviewDocRect(const QPointF &outlineDocPoint)
 {
     if (!m_showColorPreview) return QRect();
 
-    KisConfig cfg;
+    KisConfig cfg(true);
 
     const QRectF colorPreviewViewRect = cfg.colorPreviewRect();
     const QRectF colorPreviewDocumentRect = canvas()->viewConverter()->viewToDocument(colorPreviewViewRect);
@@ -703,7 +668,7 @@ void KisToolPaint::requestUpdateOutline(const QPointF &outlineDocPoint, const Ko
 {
     if (!m_supportOutline) return;
 
-    KisConfig cfg;
+    KisConfig cfg(true);
     KisPaintOpSettings::OutlineMode outlineMode;
 
     if (isOutlineEnabled() &&
