@@ -25,7 +25,7 @@
 #include <QTimer>
 #include <QCheckBox>
 
-#include <KoCanvasResourceManager.h>
+#include <KoCanvasResourceProvider.h>
 #include <KoToolBase.h>
 #include <resources/KoAbstractGradient.h>
 
@@ -110,16 +110,6 @@ protected:
         return QString();
     }
 
-    enum NodePaintAbility {
-        NONE,
-        PAINT,
-        VECTOR,
-        CLONE
-    };
-
-    /// Checks if and how the tool can paint on the current node
-    NodePaintAbility nodePaintAbility();
-
     const KoCompositeOp* compositeOp();
 
 public Q_SLOTS:
