@@ -32,10 +32,12 @@
 #include <QTimer>
 #include <QtDebug>
 #include <QDoubleSpinBox>
-
+#include "kis_cursor.h"
 #include "KisPart.h"
 #include "input/kis_input_manager.h"
 
+#include "kis_num_parser.h"
+
 class KisAbstractSliderSpinBoxPrivate {
 public:
     enum Style {
@@ -64,6 +66,8 @@ public:
     QSpinBox* dummySpinBox;
     Style style;
     bool blockUpdateSignalOnDrag;
+    bool isDragging;
+    bool parseInt;
 };
 
 KisAbstractSliderSpinBox::KisAbstractSliderSpinBox(QWidget* parent, KisAbstractSliderSpinBoxPrivate* _d)
@@ -88,11 +92,10 @@ KisAbstractSliderSpinBox::KisAbstractSliderSpinBox(QWidget* parent, KisAbstractS
     QPalette pal = d->edit->palette();
     pal.setColor(QPalette::Base, Qt::transparent);
     d->edit->setPalette(pal);
-
+    d->edit->setContextMenuPolicy(Qt::PreventContextMenu);
     connect(d->edit, SIGNAL(editingFinished()), this, SLOT(editLostFocus()));
 
     d->validator = new QDoubleValidator(d->edit);
-    d->edit->setValidator(d->validator);
 
     d->value = 0;
     d->minimum = 0;
@@ -103,8 +106,12 @@ KisAbstractSliderSpinBox::KisAbstractSliderSpinBox(QWidget* parent, KisAbstractS
     d->slowFactor = 0.1;
     d->shiftMode = false;
     d->blockUpdateSignalOnDrag = false;
+    d->isDragging = false;
+
+    d->parseInt = false;
 
     setExponentRatio(1.0);
+    setCursor(KisCursor::splitHCursor());
 
     //Set sane defaults
     setFocusPolicy(Qt::StrongFocus);
@@ -136,7 +143,6 @@ void KisAbstractSliderSpinBox::showEdit()
     d->edit->show();
     d->edit->setFocus(Qt::OtherFocusReason);
     update();
-    KisPart::currentInputManager()->slotFocusOnEnter(false);
 }
 
 void KisAbstractSliderSpinBox::hideEdit()
@@ -144,7 +150,6 @@ void KisAbstractSliderSpinBox::hideEdit()
     Q_D(KisAbstractSliderSpinBox);
     d->edit->hide();
     update();
-    KisPart::currentInputManager()->slotFocusOnEnter(true);
 }
 
 void KisAbstractSliderSpinBox::paintEvent(QPaintEvent* e)
@@ -229,7 +234,7 @@ void KisAbstractSliderSpinBox::paintFusion(QPainter &painter)
     int progressIndicatorPos = (progressOpts.progress - qreal(progressOpts.minimum)) / qMax(qreal(1.0),
                                qreal(progressOpts.maximum) - progressOpts.minimum) * rect.width();
 
-    if (progressIndicatorPos >= 0 && progressIndicatorPos <= rect.width() && (progressOpts.progress != 0)) {
+    if (progressIndicatorPos >= 0 && progressIndicatorPos <= rect.width()) {
         leftRect = QRect(rect.left(), rect.top(), progressIndicatorPos, rect.height());
     } else if (progressIndicatorPos > rect.width()) {
         painter.setPen(palette().highlightedText().color());
@@ -286,7 +291,7 @@ void KisAbstractSliderSpinBox::paintPlastique(QPainter &painter)
     int progressIndicatorPos = (progressOpts.progress - qreal(progressOpts.minimum)) / qMax(qreal(1.0),
                                qreal(progressOpts.maximum) - progressOpts.minimum) * rect.width();
 
-    if (progressIndicatorPos >= 0 && progressIndicatorPos <= rect.width() && (progressOpts.progress != 0)) {
+    if (progressIndicatorPos >= 0 && progressIndicatorPos <= rect.width()) {
         leftRect = QRect(rect.left(), rect.top(), progressIndicatorPos, rect.height());
     } else if (progressIndicatorPos > rect.width()) {
         painter.setPen(palette().highlightedText().color());
@@ -392,7 +397,6 @@ void KisAbstractSliderSpinBox::mousePressEvent(QMouseEvent* e)
         showEdit();
     }
 
-
     update();
 }
 
@@ -401,6 +405,8 @@ void KisAbstractSliderSpinBox::mouseReleaseEvent(QMouseEvent* e)
     Q_D(KisAbstractSliderSpinBox);
     QStyleOptionSpinBox spinOpts = spinBoxOptions();
 
+    d->isDragging = false;
+
     //Step up/down for buttons
     //Emualting mouse grab too
     if (upButtonRect(spinOpts).contains(e->pos()) && d->upButtonDown) {
@@ -415,7 +421,6 @@ void KisAbstractSliderSpinBox::mouseReleaseEvent(QMouseEvent* e)
     } else { // Confirm the last known value, since we might be ignoring move events
         setInternalValue(d->value);
     }
-
     d->upButtonDown = false;
     d->downButtonDown = false;
     update();
@@ -437,6 +442,7 @@ void KisAbstractSliderSpinBox::mouseMoveEvent(QMouseEvent* e)
     //Respect emulated mouse grab.
     if (e->buttons() & Qt::LeftButton &&
             !(d->downButtonDown || d->upButtonDown)) {
+        d->isDragging = true;
         setInternalValue(valueForX(e->pos().x(),e->modifiers()), d->blockUpdateSignalOnDrag);
         update();
     }
@@ -487,14 +493,41 @@ void KisAbstractSliderSpinBox::wheelEvent(QWheelEvent *e)
     e->accept();
 }
 
+bool KisAbstractSliderSpinBox::event(QEvent *event)
+{
+    if (event->type() == QEvent::ShortcutOverride){
+        QKeyEvent* key = static_cast<QKeyEvent*>(event);
+        if (key->modifiers() == Qt::NoModifier){
+            switch(key->key()){
+            case Qt::Key_Up:
+            case Qt::Key_Right:
+            case Qt::Key_Down:
+            case Qt::Key_Left:
+                event->accept();
+                return true;
+            default: break;
+            }
+        }
+    }
+    return QWidget::event(event);
+}
+
 void KisAbstractSliderSpinBox::commitEnteredValue()
 {
     Q_D(KisAbstractSliderSpinBox);
 
-    QLocale locale;
+    //QLocale locale;
     bool ok = false;
 
-    qreal value = locale.toDouble(d->edit->text(), &ok) * d->factor;
+    //qreal value = locale.toDouble(d->edit->text(), &ok) * d->factor;
+    qreal value;
+
+    if (d->parseInt) {
+        value = KisNumericParser::parseIntegerMathExpr(d->edit->text(), &ok) * d->factor;
+    } else {
+        value = KisNumericParser::parseSimpleMathExpr(d->edit->text(), &ok) * d->factor;
+    }
+
     if (ok) {
         setInternalValue(value);
     }
@@ -712,7 +745,8 @@ int KisAbstractSliderSpinBox::valueForX(int x, Qt::KeyboardModifiers modifiers)
     }
 
     //Final value
-    qreal realvalue = ((dValues * pow(percent, d->exponentRatio)) + minDbl);
+    qreal exp_percent = pow(percent, d->exponentRatio);
+    qreal realvalue = ((dValues * (percent * exp_percent >= 0 ? exp_percent : -exp_percent)) + minDbl);
     //If key CTRL is pressed, round to the closest step.
     if( modifiers & Qt::ControlModifier ) {
         qreal fstep = d->fastSliderStep;
@@ -769,11 +803,27 @@ void KisAbstractSliderSpinBox::setInternalValue(int value)
     setInternalValue(value, false);
 }
 
+bool KisAbstractSliderSpinBox::isDragging() const
+{
+    Q_D(const KisAbstractSliderSpinBox);
+    return d->isDragging;
+}
+
+void KisAbstractSliderSpinBox::setPrivateValue(int value)
+{
+    Q_D(KisAbstractSliderSpinBox);
+    d->value = qBound(d->minimum, value, d->maximum);
+}
+
 class KisSliderSpinBoxPrivate : public KisAbstractSliderSpinBoxPrivate {
 };
 
 KisSliderSpinBox::KisSliderSpinBox(QWidget* parent) : KisAbstractSliderSpinBox(parent, new KisSliderSpinBoxPrivate)
 {
+    Q_D(KisSliderSpinBox);
+
+    d->parseInt = true;
+
     setRange(0,99);
 }
 
@@ -873,6 +923,8 @@ class KisDoubleSliderSpinBoxPrivate : public KisAbstractSliderSpinBoxPrivate {
 
 KisDoubleSliderSpinBox::KisDoubleSliderSpinBox(QWidget* parent) : KisAbstractSliderSpinBox(parent, new KisDoubleSliderSpinBoxPrivate)
 {
+    Q_D(KisDoubleSliderSpinBox);
+    d->parseInt = false;
 }
 
 KisDoubleSliderSpinBox::~KisDoubleSliderSpinBox()
