@@ -40,6 +40,10 @@
 
 #include <assert.h>
 
+#if defined(KCONFIG_BEFORE_5_24)
+# define authorizeAction authorizeKAction
+#endif
+
 class KXMLGUIClientPrivate
 {
 public:
@@ -174,7 +178,7 @@ QString KXMLGUIClient::localXMLFile() const
         return QString();
     }
 
-    return QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/kxmlgui5/") +
+    return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + QStringLiteral("/kxmlgui5/") +
            componentName() + QLatin1Char('/') + d->m_xmlFile;
 }
 
@@ -234,7 +238,7 @@ void KXMLGUIClient::setXMLFile(const QString &_file, bool merge, bool setXMLDoc)
         const QString filter = componentName() + QLatin1Char('/') + _file;
 
         // files on filesystem
-        allFiles << QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kxmlgui5/") + filter); // KF >= 5.1
+        allFiles << QStandardPaths::locateAll(QStandardPaths::AppDataLocation, QStringLiteral("kxmlgui5/") + filter); // KF >= 5.1
 
         // KF >= 5.4 (resource file)
         const QString qrcFile(QStringLiteral(":/kxmlgui5/") + filter);
@@ -244,11 +248,11 @@ void KXMLGUIClient::setXMLFile(const QString &_file, bool merge, bool setXMLDoc)
 
         // then compat locations
         const QStringList compatFiles =
-                   QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, filter) + // kdelibs4, KF 5.0
-                   QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, _file); // kdelibs4, KF 5.0, caller passes component name
+                   QStandardPaths::locateAll(QStandardPaths::AppDataLocation, filter) + // kdelibs4, KF 5.0
+                   QStandardPaths::locateAll(QStandardPaths::AppDataLocation, _file); // kdelibs4, KF 5.0, caller passes component name
 
         if (allFiles.isEmpty() && !compatFiles.isEmpty()) {
-            qWarning() << "KXMLGUI file found at deprecated location" << compatFiles << "-- please use ${KXMLGUI_INSTALL_DIR} to install this file instead.";
+            qWarning() << "kxmlguiclient: KXMLGUI file found at deprecated location" << compatFiles << "-- please use ${KXMLGUI_INSTALL_DIR} to install this file instead.";
         }
         allFiles += compatFiles;
     }
@@ -429,7 +433,7 @@ bool KXMLGUIClientPrivate::mergeXML(QDomElement &base, QDomElement &additive, KA
             if (equalstr(tag, tagAction)) {
                 const QString name =  e.attribute(attrName);
                 if (!actionCollection->action(name) ||
-                        !KAuthorized::authorizeKAction(name)) {
+                        !KAuthorized::authorizeAction(name)) {
                     // remove this child as we aren't using it
                     base.removeChild(e);
                     continue;
