@@ -66,7 +66,7 @@ AnimationDocker::AnimationDocker()
     QWidget* mainWidget = new QWidget(this);
     setWidget(mainWidget);
 
-    m_animationWidget->setupUi(mainWidget);  
+    m_animationWidget->setupUi(mainWidget);
 }
 
 AnimationDocker::~AnimationDocker()
@@ -106,6 +106,9 @@ void AnimationDocker::setCanvas(KoCanvasBase * canvas)
 
 
         connect(animation, SIGNAL(sigUiTimeChanged(int)), this, SLOT(slotGlobalTimeChanged()));
+        connect(animation, SIGNAL(sigFramerateChanged()), this, SLOT(slotFrameRateChanged()));
+
+
         connect(m_canvas->animationPlayer(), SIGNAL(sigFrameChanged()), this, SLOT(slotGlobalTimeChanged()));
         connect(m_canvas->animationPlayer(), SIGNAL(sigPlaybackStopped()), this, SLOT(slotGlobalTimeChanged()));
         connect(m_canvas->animationPlayer(), SIGNAL(sigPlaybackStopped()), this, SLOT(updatePlayPauseIcon()));
@@ -116,6 +119,7 @@ void AnimationDocker::setCanvas(KoCanvasBase * canvas)
                 m_canvas->animationPlayer(),
                 SLOT(slotUpdatePlaybackSpeed(double)));
 
+
         connect(m_canvas->viewManager()->nodeManager(), SIGNAL(sigNodeActivated(KisNodeSP)),
                 this, SLOT(slotCurrentNodeChanged(KisNodeSP)));
 
@@ -135,7 +139,7 @@ void AnimationDocker::unsetCanvas()
     setCanvas(0);
 }
 
-void AnimationDocker::setMainWindow(KisViewManager *view)
+void AnimationDocker::setViewManager(KisViewManager *view)
 {
     setActions(view->actionManager());
 
@@ -220,6 +224,19 @@ void AnimationDocker::slotGlobalTimeChanged()
     m_animationWidget->intCurrentTime->setToolTip(realTimeString);
 }
 
+void AnimationDocker::slotFrameRateChanged()
+{
+    if (!m_canvas || !m_canvas->image()) return;
+
+    int fpsOnUI = m_animationWidget->intFramerate->value();
+    KisImageAnimationInterface *animation = m_canvas->image()->animationInterface();
+
+    if (animation->framerate() != fpsOnUI) {
+        m_animationWidget->intFramerate->setValue(animation->framerate());
+    }
+
+}
+
 void AnimationDocker::slotTimeSpinBoxChanged()
 {
     if (!m_canvas || !m_canvas->image()) return;
@@ -354,7 +371,7 @@ void AnimationDocker::updatePlayPauseIcon()
 
 void AnimationDocker::updateLazyFrameIcon()
 {
-    KisImageConfig cfg;
+    KisImageConfig cfg(true);
 
     const bool value = cfg.lazyFrameCreationEnabled();
 
@@ -385,7 +402,7 @@ void AnimationDocker::updateDropFramesIcon()
         isPlaying = player->isPlaying();
     }
 
-    KisConfig cfg;
+    KisConfig cfg(true);
     const bool value = cfg.animationDropFrames();
 
     m_dropFramesAction->setIcon(value ?
@@ -450,7 +467,7 @@ void AnimationDocker::slotUpdateIcons()
 
 void AnimationDocker::slotLazyFrameChanged(bool value)
 {
-    KisImageConfig cfg;
+    KisImageConfig cfg(false);
 
     if (value != cfg.lazyFrameCreationEnabled()) {
         cfg.setLazyFrameCreationEnabled(value);
@@ -460,7 +477,7 @@ void AnimationDocker::slotLazyFrameChanged(bool value)
 
 void AnimationDocker::slotDropFramesChanged(bool value)
 {
-    KisConfig cfg;
+    KisConfig cfg(false);
 
     if (value != cfg.animationDropFrames()) {
         cfg.setAnimationDropFrames(value);
@@ -601,7 +618,7 @@ void AnimationDocker::setActions(KisActionManager *actionMan)
     m_actionManager->addAction("last_frame", m_lastFrameAction);
 
     {
-        KisImageConfig cfg;
+        KisImageConfig cfg(true);
         setupActionButton(KisAnimationUtils::lazyFrameCreationActionName,
                           KisAction::ACTIVE_IMAGE,
                           cfg.lazyFrameCreationEnabled(),
@@ -610,7 +627,7 @@ void AnimationDocker::setActions(KisActionManager *actionMan)
     }
 
     {
-        KisConfig cfg;
+        KisConfig cfg(true);
         setupActionButton(KisAnimationUtils::dropFramesActionName,
                           KisAction::ACTIVE_IMAGE,
                           cfg.animationDropFrames(),
