@@ -24,19 +24,19 @@
 #include <QVariant>
 
 #include <QUrl>
-#include <kis_view_plugin.h>
+#include <KisActionPlugin.h>
 
-class Imagesplit : public KisViewPlugin
+class Imagesplit : public KisActionPlugin
 {
     Q_OBJECT
 public:
     Imagesplit(QObject *parent, const QVariantList &);
-    virtual ~Imagesplit();
+    ~Imagesplit() override;
 
 private Q_SLOTS:
 
     void slotImagesplit();
-    void saveAsImage(const QRect &imgSize, const QString &mimeType, const QString &url);
+    bool saveAsImage(const QRect &imgSize, const QString &mimeType, const QString &url);
 };
 
 #endif // IMAGESPLIT_H
