@@ -35,9 +35,10 @@ public:
     float       density;
     int         visible;
 
-    KisPaintLayer* layer;
+    KisLayer*   layer;
     KisRasterKeyframeChannel *channel;
     QString     last;
+    QString     path;
     int         frame;
 };
 
