@@ -25,7 +25,13 @@
 #include <QFileInfo>
 
 #include "kis_types.h"
-#include <KisImageBuilderResult.h>
+#include <KisImportExportErrorCode.h>
+
+
+// max number of pixels in one dimension of psd file
+extern const int MAX_PSD_SIZE;
+
+
 class KisDocument;
 
 class PSDSaver : public QObject {
@@ -35,21 +41,20 @@ class PSDSaver : public QObject {
 public:
 
     PSDSaver(KisDocument *doc);
-    virtual ~PSDSaver();
+    ~PSDSaver() override;
 
 public:
 
-    KisImageBuilder_Result buildFile(const QString &filename);
+    KisImportExportErrorCode buildFile(QIODevice *io);
 
-    KisImageWSP image();
+    KisImageSP image();
 
 public Q_SLOTS:
 
     virtual void cancel();
 
 private:
-
-    KisImageWSP m_image;
+    KisImageSP m_image;
     KisDocument *m_doc;
     bool m_stop;
 };
