@@ -21,6 +21,10 @@
 #include <compositeops/KoVcMultiArchBuildSupport.h> //MSVC requires that Vc come first
 #include "kis_auto_brush_widget.h"
 
+#include <kconfig.h>
+#include <ksharedconfig.h>
+#include <kconfiggroup.h>
+
 #include <math.h>
 #include <kis_debug.h>
 #include <QSpinBox>
@@ -34,91 +38,109 @@
 #include <kis_fixed_paint_device.h>
 #include <kis_mask_generator.h>
 #include <kis_slider_spin_box.h>
+#include "kis_signals_blocker.h"
+#include "kis_signal_compressor.h"
+#include "kis_aspect_ratio_locker.h"
 
 
 #define showSlider(input, step) input->setRange(input->minimum(), input->maximum(), step)
 #include <kis_cubic_curve.h>
 
 KisAutoBrushWidget::KisAutoBrushWidget(QWidget *parent, const char* name)
-    : KisWdgAutobrush(parent, name)
+    : KisWdgAutoBrush(parent, name)
     , m_autoBrush(0)
-    , m_linkFade(false)
+    , m_updateCompressor(new KisSignalCompressor(100, KisSignalCompressor::FIRST_ACTIVE))
+    , m_fadeAspectLocker(new KisAspectRatioLocker())
 {
+    connect(m_updateCompressor.data(), SIGNAL(timeout()), SLOT(paramChanged()));
 
-    connect(aspectButton, SIGNAL(keepAspectRatioChanged(bool)), this, SLOT(linkFadeToggled(bool)));
-    aspectButton->setKeepAspectRatio(m_linkFade);
+    connect((QObject*)comboBoxShape, SIGNAL(activated(int)), m_updateCompressor.data(), SLOT(start()));
 
-    connect((QObject*)comboBoxShape, SIGNAL(activated(int)), this, SLOT(paramChanged()));
-
-    inputRadius->setRange(0, 1000, 2);
+    inputRadius->setRange(0.01, KSharedConfig::openConfig()->group("").readEntry("maximumBrushSize", 1000), 2);
     inputRadius->setExponentRatio(3.0);
     inputRadius->setSingleStep(1);
     inputRadius->setValue(5);
     inputRadius->setSuffix(i18n(" px"));
-    connect(inputRadius, SIGNAL(valueChanged(qreal)), this, SLOT(spinBoxRadiusChanged(qreal)));
+    inputRadius->setBlockUpdateSignalOnDrag(true);
+    connect(inputRadius, SIGNAL(valueChanged(qreal)), m_updateCompressor.data(), SLOT(start()));
 
     inputRatio->setRange(0.0, 1.0, 2);
     inputRatio->setSingleStep(0.1);
     inputRatio->setValue(1.0);
-    connect(inputRatio, SIGNAL(valueChanged(qreal)), this, SLOT(spinBoxRatioChanged(qreal)));
+    inputRatio->setBlockUpdateSignalOnDrag(true);
+    connect(inputRatio, SIGNAL(valueChanged(qreal)), m_updateCompressor.data(), SLOT(start()));
 
     inputHFade->setRange(0.0, 1.0, 2);
     inputHFade->setSingleStep(0.1);
     inputHFade->setValue(0.5);
-    connect(inputHFade, SIGNAL(valueChanged(qreal)), this, SLOT(spinBoxHorizontalChanged(qreal)));
 
     inputVFade->setRange(0.0, 1.0, 2);
     inputVFade->setSingleStep(0.1);
     inputVFade->setValue(0.5);
-    connect(inputVFade, SIGNAL(valueChanged(qreal)), this, SLOT(spinBoxVerticalChanged(qreal)));
+
+    aspectButton->setKeepAspectRatio(true);
+
+    m_fadeAspectLocker->connectSpinBoxes(inputHFade, inputVFade, aspectButton);
+    m_fadeAspectLocker->setBlockUpdateSignalOnDrag(true);
+    connect(m_fadeAspectLocker.data(), SIGNAL(sliderValueChanged()), m_updateCompressor.data(), SLOT(start()));
+    connect(m_fadeAspectLocker.data(), SIGNAL(aspectButtonChanged()), m_updateCompressor.data(), SLOT(start()));
 
     inputSpikes->setRange(2, 20);
     inputSpikes->setValue(2);
-    connect(inputSpikes, SIGNAL(valueChanged(int)), this, SLOT(spinBoxSpikesChanged(int)));
+    inputSpikes->setBlockUpdateSignalOnDrag(true);
+    connect(inputSpikes, SIGNAL(valueChanged(int)), m_updateCompressor.data(), SLOT(start()));
 
     inputRandomness->setRange(0, 100);
     inputRandomness->setValue(0);
-    connect(inputRandomness, SIGNAL(valueChanged(qreal)), this, SLOT(spinBoxRandomnessChanged(qreal)));
+    inputRandomness->setBlockUpdateSignalOnDrag(true);
+    connect(inputRandomness, SIGNAL(valueChanged(qreal)), m_updateCompressor.data(), SLOT(start()));
 
     inputAngle->setRange(0, 360);
     inputAngle->setSuffix(QChar(Qt::Key_degree));
     inputAngle->setValue(0);
-    connect(inputAngle, SIGNAL(valueChanged(int)), this, SLOT(spinBoxAngleChanged(int)));
+    inputAngle->setBlockUpdateSignalOnDrag(true);
+    connect(inputAngle, SIGNAL(valueChanged(int)), m_updateCompressor.data(), SLOT(start()));
 
-    connect(spacingWidget, SIGNAL(sigSpacingChanged()), SLOT(slotSpacingChanged()));
+    connect(spacingWidget, SIGNAL(sigSpacingChanged()), m_updateCompressor.data(), SLOT(start()));
 
     density->setRange(0, 100, 0);
     density->setSingleStep(1);
     density->setValue(100);
     density->setSuffix("%");
-    connect(density, SIGNAL(valueChanged(qreal)), this, SLOT(spinBoxDensityChanged(qreal)));
+    density->setBlockUpdateSignalOnDrag(true);
+    connect(density, SIGNAL(valueChanged(qreal)), m_updateCompressor.data(), SLOT(start()));
 
     KisCubicCurve topLeftBottomRightLinearCurve;
     topLeftBottomRightLinearCurve.setPoint(0, QPointF(0.0, 1.0));
     topLeftBottomRightLinearCurve.setPoint(1, QPointF(1.0, 0.0));
     softnessCurve->setCurve(topLeftBottomRightLinearCurve);
-    connect(softnessCurve, SIGNAL(modified()), this, SLOT(paramChanged()));
+    connect(softnessCurve, SIGNAL(modified()), m_updateCompressor.data(), SLOT(start()));
 
     m_brush = QImage(1, 1, QImage::Format_RGB32);
 
-    connect(brushPreview, SIGNAL(clicked()), SLOT(paramChanged()));
+    connect(brushPreview, SIGNAL(clicked()), m_updateCompressor.data(), SLOT(start()));
 
     QList<KoID> ids = KisMaskGenerator::maskGeneratorIds();
     for (int i = 0; i < ids.size(); i++) {
         comboBoxMaskType->insertItem(i, ids[i].name());
     }
 
-    connect(comboBoxMaskType, SIGNAL(activated(int)), SLOT(paramChanged()));
+    connect(comboBoxMaskType, SIGNAL(activated(int)), m_updateCompressor.data(), SLOT(start()));
     connect(comboBoxMaskType, SIGNAL(currentIndexChanged(int)), SLOT(setStackedWidget(int)));
+    setStackedWidget(comboBoxMaskType->currentIndex());
 
     brushPreview->setIconSize(QSize(100, 100));
 
-    connect(btnAntialiasing, SIGNAL(toggled(bool)), SLOT(paramChanged()));
+    connect(btnAntialiasing, SIGNAL(toggled(bool)), m_updateCompressor.data(), SLOT(start()));
 
-    paramChanged();
+    m_updateCompressor->start();
 
 }
 
+KisAutoBrushWidget::~KisAutoBrushWidget()
+{
+}
+
 void KisAutoBrushWidget::resizeEvent(QResizeEvent *)
 {
     brushPreview->setMinimumHeight(brushPreview->width()); // dirty hack !
@@ -127,7 +149,7 @@ void KisAutoBrushWidget::resizeEvent(QResizeEvent *)
 
 void KisAutoBrushWidget::activate()
 {
-    paramChanged();
+    m_updateCompressor->start();
 }
 
 void KisAutoBrushWidget::paramChanged()
@@ -167,6 +189,12 @@ void KisAutoBrushWidget::paramChanged()
     m_autoBrush->setAutoSpacing(spacingWidget->autoSpacingActive(), spacingWidget->autoSpacingCoeff());
     m_brush = m_autoBrush->image();
 
+    drawBrushPreviewArea();
+
+    emit sigBrushChanged();
+}
+
+void KisAutoBrushWidget::drawBrushPreviewArea() {
     QImage pi(m_brush);
     double coeff = 1.0;
     int bPw = brushPreview->width() - 3;
@@ -183,8 +211,6 @@ void KisAutoBrushWidget::paramChanged()
 
     QPixmap p = QPixmap::fromImage(pi);
     brushPreview->setIcon(QIcon(p));
-
-    emit sigBrushChanged();
 }
 
 void KisAutoBrushWidget::setStackedWidget(int index)
@@ -197,95 +223,6 @@ void KisAutoBrushWidget::setStackedWidget(int index)
     }
 }
 
-void KisAutoBrushWidget::spinBoxHorizontalChanged(qreal a)
-{
-    if (m_linkFade) {
-        inputVFade->blockSignals(true);
-        inputVFade->setValue(a);
-        inputVFade->blockSignals(false);
-    }
-
-    inputHFade->blockSignals(true);
-    inputHFade->setValue(a);
-    inputHFade->blockSignals(false);
-    paramChanged();
-}
-
-void KisAutoBrushWidget::spinBoxVerticalChanged(qreal a)
-{
-    if (m_linkFade) {
-        inputHFade->blockSignals(true);
-        inputHFade->setValue(a);
-        inputHFade->blockSignals(false);
-    }
-
-    inputVFade->blockSignals(true);
-    inputVFade->setValue(a);
-    inputVFade->blockSignals(false);
-    paramChanged();
-}
-
-void KisAutoBrushWidget::spinBoxRatioChanged(qreal a)
-{
-    inputRatio->blockSignals(true);
-    inputRatio->setValue(a);
-    inputRatio->blockSignals(false);
-    paramChanged();
-}
-
-void KisAutoBrushWidget::spinBoxRandomnessChanged(qreal a)
-{
-    inputRandomness->blockSignals(true);
-    inputRandomness->setValue(a);
-    inputRandomness->blockSignals(false);
-    paramChanged();
-}
-
-void KisAutoBrushWidget::spinBoxRadiusChanged(qreal a)
-{
-    inputRadius->blockSignals(true);
-    inputRadius->setValue(a);
-    inputRadius->blockSignals(false);
-    paramChanged();
-}
-
-void KisAutoBrushWidget::spinBoxSpikesChanged(int a)
-{
-    inputSpikes->blockSignals(true);
-    inputSpikes->setValue(a);
-    inputSpikes->blockSignals(false);
-    paramChanged();
-}
-
-void KisAutoBrushWidget::spinBoxAngleChanged(int a)
-{
-    inputAngle->blockSignals(true);
-    inputAngle->setValue(a);
-    inputAngle->blockSignals(false);
-    paramChanged();
-}
-
-void KisAutoBrushWidget::spinBoxDensityChanged(qreal a)
-{
-    density->blockSignals(true);
-    density->setValue(a);
-    density->blockSignals(false);
-    paramChanged();
-}
-
-void KisAutoBrushWidget::slotSpacingChanged()
-{
-    paramChanged();
-}
-
-void KisAutoBrushWidget::linkFadeToggled(bool b)
-{
-    m_linkFade = b;
-
-    if (m_linkFade)
-        inputVFade->setValue(inputHFade->value());
-}
-
 KisBrushSP KisAutoBrushWidget::brush()
 {
     return m_autoBrush;
@@ -297,6 +234,11 @@ void KisAutoBrushWidget::setBrush(KisBrushSP brush)
     m_brush = brush->image();
     // XXX: lock, set and unlock the widgets.
     KisAutoBrush* aBrush = dynamic_cast<KisAutoBrush*>(brush.data());
+
+    KisSignalsBlocker b1(comboBoxShape, comboBoxMaskType);
+    KisSignalsBlocker b2(inputRadius, inputRatio, inputHFade, inputVFade, inputAngle, inputSpikes);
+    KisSignalsBlocker b3(spacingWidget, inputRandomness, density, softnessCurve, btnAntialiasing);
+
     if (aBrush->maskGenerator()->type() == KisMaskGenerator::CIRCLE) {
         comboBoxShape->setCurrentIndex(0);
     }
@@ -307,7 +249,9 @@ void KisAutoBrushWidget::setBrush(KisBrushSP brush)
         comboBoxShape->setCurrentIndex(2);
     }
 
-    comboBoxMaskType->setCurrentIndex(comboBoxMaskType->findText(aBrush->maskGenerator()->name()));
+    const int mastTypeIndex = comboBoxMaskType->findText(aBrush->maskGenerator()->name());
+    comboBoxMaskType->setCurrentIndex(mastTypeIndex);
+    setStackedWidget(mastTypeIndex); // adjusting manually because the signals are blocked
 
     inputRadius->setValue(aBrush->maskGenerator()->diameter());
     inputRatio->setValue(aBrush->maskGenerator()->ratio());
@@ -328,6 +272,8 @@ void KisAutoBrushWidget::setBrush(KisBrushSP brush)
     }
 
     btnAntialiasing->setChecked(aBrush->maskGenerator()->antialiasEdges());
+
+    drawBrushPreviewArea(); // sync up what the brush preview area looks like
 }
 
 
