@@ -28,8 +28,8 @@ class MergeEffectFactory : public KoFilterEffectFactoryBase
 {
 public:
     MergeEffectFactory();
-    virtual KoFilterEffect *createFilterEffect() const;
-    virtual KoFilterEffectConfigWidgetBase *createConfigWidget() const;
+    KoFilterEffect *createFilterEffect() const override;
+    KoFilterEffectConfigWidgetBase *createConfigWidget() const override;
 };
 
 #endif // MERGEEFFECTFACTORY_H
